package infomat;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.*;


/**
 * Writes the default Properties of several classes to a directory.
 * Good as a start for setting up experiments. Check that all classes
 * you need are included.
 * <br>
 *
 * @version          081001
 * @author           Magnus Rosell
 *
 *
 * java -cp classes/ -Xms1024m -Xmx1024m infomat.WriteProperties <resultPath>
 */
public class WriteProperties{
    private static void badInput(){
	System.out.println("Bad input!");
	System.out.println();
	System.out.println("java infomat.WriteProperties <resultPath>");
	System.exit(0);
    }

    // "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/InfomatRes/"

    public static void main(String args[]) throws IOException{

	if(args.length!=1)
	    badInput();
	String resultPath=args[0];

	if(!resultPath.endsWith(File.separator))
	    resultPath += File.separator;

	/**********************************/
	File propertiesDirectory = null; // dummy
	Experimentator exp = new Experimentator(propertiesDirectory);
	exp.getProperties().saveToDirectory(resultPath);

	PropertiesPool pool = null; // dummy
	Clusterer clusterer = new Clusterer(pool);
	clusterer.getProperties().saveToDirectory(resultPath);

	/**********************************/
	IMatrix matrix = new IMatrix("Dummy rows", "Dummy cols");
	boolean row = true;

	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);
	dotSim.getProperties().saveToDirectory(resultPath);

	SparseOverlapSimilarity overlapSim = new SparseOverlapSimilarity(matrix, row);
	overlapSim.getProperties().saveToDirectory(resultPath);

	/**********************************/
	KMeans kmeans = new KMeans(dotSim);
	kmeans.getProperties().saveToDirectory(resultPath);

	BisectingKMeans bisectingKMeans = new BisectingKMeans();
	bisectingKMeans.getProperties().saveToDirectory(resultPath);

	LocationGrouper lg = new LocationGrouper();
	lg.getProperties().saveToDirectory(resultPath);

	RandomClusterer rc = new RandomClusterer();
	rc.getProperties().saveToDirectory(resultPath);

	RelativeClusterer relc = new RelativeClusterer();
	relc.getProperties().saveToDirectory(resultPath);

	/**********************************/
	Stoplist stoplist = new Stoplist();
	stoplist.getProperties().saveToDirectory(resultPath);

	/**********************************/
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.getProperties().saveToDirectory(resultPath);

	/**********************************/
	TFIDFWeighter tfidfWeighter = new TFIDFWeighter();
	tfidfWeighter.getProperties().saveToDirectory(resultPath);

	TFWeighter tfWeighter = new TFWeighter();
	tfWeighter.getProperties().saveToDirectory(resultPath);

	CountWeighter countWeighter = new CountWeighter();
	countWeighter.getProperties().saveToDirectory(resultPath);

	/**********************************/
	IObjectGroupingIO iogingio = new IObjectGroupingIO();
	iogingio.getProperties().saveToDirectory(resultPath);

	/**********************************/
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.getProperties().saveToDirectory(resultPath);

	/**********************************/
	IObjectGrouping evalGrouping = null; // dummy
	IObjectGrouping refGrouping = null; // dummy
	ISimilarity similarity = null; // dummy
	String stamp = null; // dummy
	Evaluation evaluation = new Evaluation(matrix, evalGrouping, refGrouping, similarity, stamp);
	evaluation.getProperties().saveToDirectory(resultPath);

	/**********************************/

	/*
	LabelReducer lbred = new LabelReducer();
	lbred.getProperties().saveToDirectory(resultPath);
	//	lbred.getProperties().save(resultPath+"labelReducer.xml");

	CentroidLabeler centroidLabeler = new CentroidLabeler();
	centroidLabeler.getProperties().saveToDirectory(resultPath);

	RelativeLabeler relativeLabeler = new RelativeLabeler(matrix, row);
	relativeLabeler.getProperties().saveToDirectory(resultPath);

	WeightRankingBoostLabeler weightRankingBoostLabeler = 
	    new WeightRankingBoostLabeler(tfidfWeighter, matrix, row);
	weightRankingBoostLabeler.getProperties().saveToDirectory(resultPath);
	//	weightRankingBoostLabeler.getProperties().save(resultPath+"weightRankingBoostLabeler.xml");

	CentroidRankingBoostLabeler centroidRankingBoostLabeler = 
	    new CentroidRankingBoostLabeler();
	centroidRankingBoostLabeler.getProperties().saveToDirectory(resultPath);

	CentroidBoostLabeler centroidBoostLabeler = 
	    new CentroidBoostLabeler();
	centroidBoostLabeler.getProperties().saveToDirectory(resultPath);

	CombinedCentroidLabeler combinedCentroidLabeler = 
	    new CombinedCentroidLabeler();
	combinedCentroidLabeler.getProperties().saveToDirectory(resultPath);

	//	centroidRankingBoostLabeler.getProperties().save(resultPath+"centroidRankingBoostLabeler.xml");
	*/

	/*
	SizeReducer sizered = new SizeReducer();
	sizered.getProperties().save(resultPath+"sizeReducer.xml");
	*/

    }
}
