package infomat.algorithms.labeling;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 */
public class CentroidRankingBoostLabeler extends Labeler{

    public CentroidRankingBoostLabeler(){
	this.properties = getDefaultProperties();
    }



    // Gr hjlpklass i Properties-paketet 
    // som gr att man kan stt properties frn 
    // Array:er med Object.


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("CentroidRankingBoostLabeler Properties", CurrentClassGetter.getClassName());
		
	try{


	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    //    public static int numberOfObjects=10;

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /**
     * Can not be used separately, since it relyes on having information 
     * on the parent group.
     */
    private ILabel labelGroup(IObjectGroup iog, SparseISimilarity similarity, HashMap<IObject, Float> wholeGroupingWeightOrderMap){

	//	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	//	    iog.getSortedCentroid(matrix);

	/*
	HashMap<IObject, Float> map = similarity.getGroupRepresentation(iog);
	IObjectFloatMapEntryComparator entryComparator = 
	    new IObjectFloatMapEntryComparator(map);
	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	    new Vector<Map.Entry<IObject, Float>>(map.entrySet());
	Collections.sort(objWeightVec, entryComparator);
	*/

	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	    getSortedGroupRepresentation(iog, similarity);


	HashMap<IObject, Float> weightMap = 
	    new HashMap<IObject, Float>();
	int rank=1;
	for(int i=0; i<objWeightVec.size(); i++){
	    IObject io = objWeightVec.elementAt(i).getKey();
	    float wholeRank = 
		wholeGroupingWeightOrderMap.get(io);
	    float labelWeight = (float) 
		(MoreMath.log2(wholeRank) - MoreMath.log2(rank));
	    weightMap.put(io, labelWeight);
	    rank++;
	}
	
	// Sortera p labelWeight inom gruppen
	IObjectWeightComparator owc = new IObjectWeightComparator(weightMap);
	TreeMap<IObject, Float> orderedWeightMap = 
	    new TreeMap<IObject, Float>(owc);
	orderedWeightMap.putAll(weightMap);

	ILabel label = new ILabel(iog);
	int numberOfObjects = (Integer) properties.getPropertyValue("Number of objects in labels");
	Iterator<Map.Entry<IObject, Float>> it = 
	    orderedWeightMap.entrySet().iterator();
	for(int i=0; i<numberOfObjects && it.hasNext(); i++){
	    Map.Entry<IObject, Float> entry = it.next();
	    IObject io = entry.getKey();
	    float labelWeight = entry.getValue();
	    label.add(io, labelWeight);
	}

	return label;
    }



    public HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity){

	HierarchicalIObjectGrouping hioging = null;

	if(ioging instanceof HierarchicalIObjectGrouping){
	    hioging = (HierarchicalIObjectGrouping) ioging;

	}else{
	    hioging = new HierarchicalIObjectGrouping("Temporary HierarchicalIObjectGrouping");
	    IObjectGroupNode root = new IObjectGroupNode(ioging, null);
	    Vector<IObjectGroupNode> nodeVec = new Vector<IObjectGroupNode>();
	    nodeVec.add(root);
	    hioging.setRootIObjectGroupNodes(nodeVec);

	    nodeVec = new Vector<IObjectGroupNode>();
	    Vector<IObjectGroup> groups = ioging.getGroups();
	    IObjectGroupNode parent = new IObjectGroupNode(ioging, null);
	    for(int i=0; i<groups.size(); i++){
		IObjectGroup iog = groups.elementAt(i);
		IObjectGroupNode node = new IObjectGroupNode(iog, parent);
		nodeVec.add(node);
	    }
	    hioging.setBottomIObjectGroupNodes(nodeVec);
	}

	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();
	    
	HashMap<IObjectGroupNode, HashMap<IObject, Float>> parentMapMap =
	    new HashMap<IObjectGroupNode, HashMap<IObject, Float>>();

	Vector<IObjectGroupNode> bottomNodes = 
	    hioging.getBottomIObjectGroupNodes();

	for(int i=0; i<bottomNodes.size(); i++){
	    IObjectGroupNode node = bottomNodes.elementAt(i);
	    IObjectGroupNode parent = node.getParent();

	    HashMap<IObject, Float> objWeightOrderMap = 
		parentMapMap.get(parent);
	    if(objWeightOrderMap==null){
		Vector<Map.Entry<IObject, Float>> objWeightVec = 
		    getSortedGroupRepresentation(parent.getIObjectGroup(), similarity);
		objWeightOrderMap = new HashMap<IObject, Float>();
		int rank=1;
		for(int j=0; j<objWeightVec.size(); j++){
		    IObject io = objWeightVec.elementAt(j).getKey();
		    objWeightOrderMap.put(io, (float) rank);
		    rank++;
		}
		parentMapMap.put(parent, objWeightOrderMap);
	    }

	    IObjectGroup iog = node.getIObjectGroup();
	    ILabel label = 
		labelGroup(iog, similarity, objWeightOrderMap);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;


	/*
	//	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	//	    ioging.getSortedCentroid(matrix);
	HashMap<IObject, Float> map = 
	similarity.getGroupRepresentation(ioging);
	IObjectFloatMapEntryComparator entryComparator = 
	new IObjectFloatMapEntryComparator(map);
	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	new Vector<Map.Entry<IObject, Float>>(map.entrySet());
	Collections.sort(objWeightVec, entryComparator);

	HashMap<IObject, Float> objWeightOrderMap = 
	new HashMap<IObject, Float>();
	int rank=1;
	for(int i=0; i<objWeightVec.size(); i++){
	IObject io = objWeightVec.elementAt(i).getKey();
	objWeightOrderMap.put(io, (float) rank);
	rank++;
	}


	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	new HashMap<IObjectGroup, ILabel>();

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	IObjectGroup iog = groups.elementAt(i);
	ILabel label = 
	labelGroup(iog, similarity, objWeightOrderMap);
	groupLabelMap.put(iog, label);
	}

	return groupLabelMap;
	*/
    }




}

