package infomat.algorithms.labeling;

import infomat.algorithms.grouping.NeedsIMatrix;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 */
public class WeightRankingBoostLabeler extends Labeler{

    private Weighter weighter;
    private IMatrix matrix;
    private boolean row;

    public WeightRankingBoostLabeler(Weighter weighter, IMatrix matrix, boolean row){
	this.properties = getDefaultProperties();
	this.weighter = weighter;
	this.matrix = matrix;
	this.row=row;
    }

    public WeightRankingBoostLabeler(mro.util.properties.Properties properties, Weighter weighter, IMatrix matrix, boolean row){
	this.properties = properties;
	this.weighter = weighter;
	this.matrix = matrix;
	this.row=row;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("WeightRankingBoostLabeler Properties", CurrentClassGetter.getClassName());
		
	try{


	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    //    public static int numberOfObjects=10;

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /**
     * Can not be used separately, since it relyes on having information 
     * on the whole set.
     */
    private ILabel labelGroup(IObjectGroup iog, SparseISimilarity similarity, HashMap<IObject, Float> wholeGroupingWeightOrderMap, Weighter weighter){

	/*
	boolean row = similarity.getIMatrixDimension();
	IMatrix matrix = similarity.getIMatrix();
	*/
	IObjectGroup rowGroup = null;
	IObjectGroup colGroup = null;
	if(row){
	    rowGroup = iog;
	    colGroup = matrix.constructCorrespondingGroup(iog);
	}else{
	    rowGroup = matrix.constructCorrespondingGroup(iog);
	    colGroup = iog;
	}

	/*
	boolean row = false;
	IObjectGroup correspondingGroup = 
	    matrix.constructCorrespondingGroup(iog);

	IObjectGroup rowGroup = correspondingGroup;
	IObjectGroup colGroup = iog;
	if(matrix.rowObject(iog)){
	    rowGroup = iog;
	    colGroup = correspondingGroup;
	    row=true;
	}
	*/

	IMatrix subMatrix = matrix.subMatrix(rowGroup, colGroup);
	weighter.weight(subMatrix);

	boolean localWeight = false;
	HashMap<IObject, Float> weightMap = 
	    subMatrix.getTotalWeight(row, localWeight);

	// Soretera gruppens vikter
	IObjectWeightComparator owc = new 
	    IObjectWeightComparator(weightMap);
	TreeMap<IObject, Float> orderedWeightMap = 
	    new TreeMap<IObject, Float>(owc);
	orderedWeightMap.putAll(weightMap);
	
	// Ranka om
	weightMap = new HashMap<IObject, Float>();
	int rank=1;
	Iterator<IObject> it2 = 
	    orderedWeightMap.keySet().iterator();
	while(it2.hasNext()){
	    IObject io = it2.next();
	    float wholeRank = 
		wholeGroupingWeightOrderMap.get(io);
	    float newRank = (float) 
		(MoreMath.log2(wholeRank) - MoreMath.log2(rank));
	    weightMap.put(io, newRank);
	    rank++;
	}
	
	// Sortera p ranking inom gruppen
	owc = new IObjectWeightComparator(weightMap);
	orderedWeightMap = new TreeMap<IObject, Float>(owc);
	orderedWeightMap.putAll(weightMap);
	
	ILabel label = new ILabel(iog);
	Iterator<Map.Entry<IObject, Float>> it = 
	    orderedWeightMap.entrySet().iterator();
	int numberOfObjects = (Integer) properties.getPropertyValue("Number of objects in labels");
	int i=0;
	while(it.hasNext() && i<numberOfObjects){
	    Map.Entry<IObject, Float> entry = it.next();
	    label.add(entry.getKey(), entry.getValue());
	    i++;
	}
	return label;


    }


    public HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity){
	
	if(!(similarity instanceof NeedsIMatrix)){
	    System.out.println("WeightRankingBoostLabeler needs a SparseISimilarity that implements NeedsIMatrix!!!");
	    System.exit(0);
	}

	/*	
	boolean row = similarity.getIMatrixDimension();
	IMatrix matrix = ((NeedsIMatrix) similarity).getIMatrix();
	*/
	IObjectGroup rowGroup = null;
	IObjectGroup colGroup = null;
	if(row){
	    rowGroup = ioging;
	    colGroup = matrix.constructCorrespondingGroup(ioging);
	}else{
	    rowGroup = matrix.constructCorrespondingGroup(ioging);
	    colGroup = ioging;
	}

	/*
	boolean row = false;
	IObjectGroup correspondingGroup = 
	    matrix.constructCorrespondingGroup(ioging);

	IObjectGroup rowGroup = correspondingGroup;
	IObjectGroup colGroup = ioging;
	if(matrix.rowObject(ioging)){
	    rowGroup = ioging;
	    colGroup = correspondingGroup;
	    row=true;
	}
	*/

	IMatrix subMatrix = matrix.subMatrix(rowGroup, colGroup);
	weighter.weight(subMatrix);

	boolean localWeight = false;
	HashMap<IObject, Float> weightMap = 
	    subMatrix.getTotalWeight(row, localWeight);

	IObjectWeightComparator owc = new 
	    IObjectWeightComparator(weightMap);
	TreeMap<IObject, Float> orderedWeightMap = 
	    new TreeMap<IObject, Float>(owc);
	orderedWeightMap.putAll(weightMap);
	
	int rank=1;
	Iterator<IObject> it2 = 
	    orderedWeightMap.keySet().iterator();
	while(it2.hasNext()){
	    IObject io = it2.next();
	    weightMap.put(io, (float) rank);
	    rank++;
	}

	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);
	    ILabel label = 
		labelGroup(iog, similarity, weightMap, weighter);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;



    }




}

