package infomat.bridges.jsdm;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For importing a Random Index
 * from Martin Hassel's package
 * java sdm (jsdm)
 * see: http://www.nada.kth.se/~xmartin/
 * <br><br>
 * This is a naive implementation. Each index of the vectors
 * is treated as an IObject. The importation is from the
 * result files of the jsdm, so there is no need to have the
 * package installed. You'll need jsdm result files, though :).
 * 
 * @version          080605
 * @author           Magnus Rosell
 */
public class RIIO implements HasProperties{
    mro.util.properties.Properties properties;

    public RIIO(){
	this.properties = getDefaultProperties();
    }


    public RIIO(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Random Indexing Import Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Least Term Frequency", 10, "Minimum Term Frequency to be Imported");
	    properties.addProperty(p);
	    p = new Property("Least Document Frequency", 0, "Minimum Document Frequency to be Imported");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }







    /**
     * Loads a IMatrix from file in XML format
     */
    public IMatrix load(File file) {
	System.out.println("************** "+file.getPath());
	try {
	    BufferedReader in = 
		new BufferedReader(new FileReader(file));
	    InputSource source = new InputSource(in);
	    return load(source);
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;
    }

    public IMatrix loadCompressed(File file) {
	try {
	    // Create a ZipInputStream to read the zip file            
	    ZipInputStream zis = new ZipInputStream(new BufferedInputStream(new FileInputStream(file)));
	    InputSource source = new InputSource(zis);
	    ZipEntry entry;
	    if((entry = zis.getNextEntry()) != null) {
		if(!entry.isDirectory()) {
		    return load(source);
		}
	    }
	    zis.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return null;

    }




    /**
     * Load RandomIndex from file.
     * @param source InputSource from (compressed) file, URL etc.
     * @return number of <code>RandomLabel</code>s loaded, or <code>null</code> on failure.
     */
    private IMatrix load(InputSource source) {
	IMatrix matrix = null;

        try {

	    int leastTF = (Integer) getPropertyValue("Least Term Frequency");
	    int leastDF = (Integer) getPropertyValue("Least Document Frequency");

	    matrix = new IMatrix("Dimensions", "Words");

	    /*
            RIHandler handler = new RIHandler(matrix);
	    SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            parser.parse(source, handler);
	    */

	    XMLReader parser = XMLReaderFactory.createXMLReader();
            RIHandler handler = new RIHandler(matrix, leastTF, leastDF);
	    parser.setContentHandler(handler);
	    parser.parse(source);

	    /*
            // Set internal variables from values read into the XML labelHandler
            _randomIndex = handler.randomIndex;
	    _vocabulary = handler.vocabulary;
            _dimensionality = handler.dimensionality;
            _randomDegree = handler.randomDegree;
            _seed = handler.randomSeed;
            _leftWindowSize = handler.leftWindowSize;
            _rightWindowSize = handler.rightWindowSize;
            try {
                _weightingScheme = (WeightingScheme)Class.forName(handler.weightingScheme).newInstance();
            } catch (ClassNotFoundException ex) {
                System.err.println(ex);
                ex.printStackTrace();
            }
            _wordsIndexed = handler.wordsIndexed;
            _documentsIndexed = handler.documentsIndexed;
            _allLowerCase = handler.allLowercase;
            _unaryLabels = handler.unaryLabels;
            _documentLabels = handler.documentLabels;
            _createAhead = handler.createAhead;
            _purged = handler.purged;
            return _randomIndex.size();
	    */

        } catch (Exception ex) {
            System.err.println(ex);
            ex.printStackTrace();
        }


        return matrix;
    }




    public class RIHandler extends DefaultHandler {
	// These are per RandomIndex, used as 'return values' and therefore public
	//	public ConcurrentHashMap<String,RandomLabel> randomIndex = new ConcurrentHashMap<String,RandomLabel>();
	public Set<String> vocabulary = new HashSet<String>();
	public int dimensionality = 0;
	public int randomDegree = 0;
	public int randomSeed = 0;
	public int leftWindowSize = 0;
	public int rightWindowSize = 0;
	public String weightingScheme = "";
	public long wordsIndexed = 0;
	public long documentsIndexed = 0;
	public boolean allLowercase = true;
	public boolean unaryLabels = false;
	public boolean documentLabels = false;
	public boolean createAhead = true;
	public boolean purged = false;
	// These are per RandomLabel read from the XML
	private String _word;	// The word the 'label' is to be attached to
	private String _meta_data;	 // Field available for meta data (e.g. PoS?)
	private long _termfrequency; // Number of updates done to the label
	private int  _docfrequency;	 // Number of documents the word occurs in
	private int _negs[];	// Posititions for -1:s in the random label
	private int _poss[];	// Posititions for 1:s in the random label
	private float _context[];	// The actual contextually updated label
	// Buffer to read chunk data into
	private StringBuffer buffer = null;
	private boolean progressIndicator = false;
	private boolean _inVocabulary = false;
	private boolean _foundCV = false;
	private int no_pos_and_neg;


	private IMatrix matrix;
	private IObjectSet colSet;
	private IObjectSet rowSet;

	private int leastTF;
	private int leastDF;
	
	private int id = 0;


	//	PrintWriter pw = null;


	public RIHandler(IMatrix matrix, int leastTF, int leastDF){
	    this.matrix = matrix;
	    colSet = matrix.getColumnIObjectSet();
	    rowSet = matrix.getColumnIObjectSet();

	    this.leastTF=leastTF;
	    this.leastDF=leastDF;

	    /*
	    try{
		pw = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(new File("/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/ristrings.txt")))));
	    }catch(IOException ioe){
		// Sue me
	    }
	    */

	}

	public void startElement(String namespaceURI, String localName,
				 String qualifiedName, Attributes attributes) {
	    buffer = new StringBuffer();

	    // <randomlabels>
	    if(qualifiedName.equals("randomlabels")) {
		System.out.print("Reading Random Index: .");
		dimensionality = Integer.parseInt(attributes.getValue("dimensionality"));

		for(id=0; id<dimensionality; id++){
		    IObject featureIO = new IObject("ri"+id, id);
		    matrix.addColumnIObject(featureIO);
		}


		randomDegree = Integer.parseInt(attributes.getValue("randomness"));
		randomSeed = Integer.parseInt(attributes.getValue("seed"));
		leftWindowSize = Integer.parseInt(attributes.getValue("leftWindowSize"));
		rightWindowSize = Integer.parseInt(attributes.getValue("rightWindowSize"));
		weightingScheme = attributes.getValue("weightingScheme");
		documentsIndexed = Long.parseLong(attributes.getValue("documentsIndexed"));
		if(attributes.getValue("allLowercase")!=null)
		    allLowercase = (attributes.getValue("allLowercase").compareToIgnoreCase("true")==0) ? true : false;
		if(attributes.getValue("unaryLabels")!=null)
		    unaryLabels = (attributes.getValue("unaryLabels").compareToIgnoreCase("true")==0) ? true : false;
		if(attributes.getValue("documentLabels")!=null)
		    documentLabels = (attributes.getValue("documentLabels").compareToIgnoreCase("true")==0) ? true : false;
		if(attributes.getValue("createAhead")!=null)
		    createAhead = (attributes.getValue("createAhead").compareToIgnoreCase("true")==0) ? true : false;
		if(attributes.getValue("purged")!=null)
		    purged = (attributes.getValue("purged").compareToIgnoreCase("true")==0) ? true : false;
		// Unused so far but enhances the informativeness of the XML to human eyes
		// Could be used for content validating beyond well-formedness
		// Long.parseLong(atts.getValue("wordsIndexed"));
		// Long.parseLong(atts.getValue("labels"));
		no_pos_and_neg = Math.round(randomDegree/2);
	    }

	    // <rl>
	    if(qualifiedName.equals("rl")) {
		_word = "";
		_meta_data = null;
		_termfrequency = 0;
		_docfrequency = 0;
    		_negs = new int[no_pos_and_neg];
    		_poss = new int[no_pos_and_neg];
		_context = new float[dimensionality];
	    }

	    // <vocabulary>
	    if(qualifiedName.equals("vocabulary"))
		_inVocabulary = true;
	}

	public void endElement(String namespaceURI, String localName, String qualifiedName) {
	    String chunk = buffer.toString();

	    // </vocabulary>
	    if(qualifiedName.equals("vocabulary"))
		_inVocabulary = false;

	    // </wd>
	    if(qualifiedName.equals("wd")) {
		_word = chunk;
		//		_word = TagStripper.fromXMLentities(chunk);

		//		System.out.println(_word);
		//		pw.println(_word);

		if(_inVocabulary)
		    vocabulary.add(_word);
	    }

	    // </md>
	    if(qualifiedName.equals("md")){
		
		_meta_data = null;
		if(chunk.length()>0)
		    _meta_data = chunk;
		    

		//		_meta_data = chunk.length()>0 ? TagStripper.fromXMLentities(chunk) : null;

	    }
	    // </tf>
	    if(qualifiedName.equals("tf"))
		_termfrequency = Long.parseLong(chunk);

	    // </df>
	    if(qualifiedName.equals("df"))
		_docfrequency  = Integer.parseInt(chunk);

	    // </neg>
	    if(qualifiedName.equals("neg")) {
        	if(chunk.length() > 0) {
		    String data_array[] = chunk.split(",");
		    for(int i = 0; i < data_array.length; i++)
			_negs[i] = Integer.parseInt(data_array[i]);
        	}
	    }

	    // </pos>
	    if(qualifiedName.equals("pos")) {
        	if(chunk.length() > 0) {
		    String data_array[] = chunk.split(",");
		    for(int i = 0; i < data_array.length; i++)
			_poss[i] = Integer.parseInt(data_array[i]);
        	}
	    }

	    // </cv>
	    if(qualifiedName.equals("cv")) {
        	if(chunk.length() > 0) {
		    String data_array[] = chunk.split(",");
		    for(int i = 0; i < data_array.length; i++)
			_context[i] = Float.valueOf(data_array[i]).floatValue();
		    _foundCV = true;
        	}
	    }

	    // </rl>
	    if(qualifiedName.equals("rl")) {

		if(_termfrequency >= leastTF && _docfrequency >= leastDF){
		
		    IObject wordIO = new IObject(_word, id);
		    id++;
		    
		    if(id%100==0)
			System.out.print(".");

		    matrix.addRowIObject(wordIO);
		    
		    for(int i=0; i<_context.length; i++){
			IObject featureIO = colSet.getIObjectAt(i);
			
			IMatrixCell mc = 
			    new IMatrixCell(wordIO, featureIO, _context[i]);
			matrix.addCell(mc);
		    }
		}


		/*
		RandomLabel rlabel = new RandomLabel(_word, _meta_data, 
						     _termfrequency, _docfrequency, _negs, _poss, _context);
        	// If no context vector was to be found then the label should be pruned
        	if(!_foundCV)
		    rlabel.prune();
    		_foundCV = false;
		randomIndex.put(_word, rlabel);
		wordsIndexed += _termfrequency;
		if(progressIndicator)
		    System.out.print(".");
		*/
	    }

	    if(qualifiedName.equals("randomlabels")) {
		System.out.println();
	    }

	}

	public void characters(char[] ch, int start, int length) {
	    if(buffer != null)
		buffer.append(ch, start, length);
	}

	public void endDocument() {
	    if(progressIndicator)
		System.out.print("\n");
	}

    }



}
