package infomat;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;



/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp classes/ infomat.ExampleClusterer</code>
 *
 * @version          080605
 * @author           Magnus Rosell
 *
 */
public class Test{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	/*
	String matrixFileName = "examples/data/eng_matrix.xml";
	File matrixFile = new File(matrixFileName);
	IMatrixIO iomatrix = new IMatrixIO();
	IMatrix matrix = iomatrix.load(matrixFile);
	*/

	String tokenFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/ATot20.randomPart1.corpus.tokenFile.xml";
	File tokenFile = new File(tokenFileName);
	TokenFileTextWordIO tfio = new TokenFileTextWordIO();
	IMatrix matrix = tfio.load(tokenFile);



	// Load a categorization
	/*
	String categorizationFileName = "examples/data/eng_text_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	*/

	// Filter the matrix
	// The example is to small
	/*
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);
	*/

	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Cluster the rows (texts) of the matrix
	IObjectSet rowSet = matrix.getRowIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Cluster
	KMeans kmeans = new KMeans(dotSim);
	try{
	    kmeans.setProperty("Repeat number of times", 1);
	}catch(PropertyNotFoundException pnfe){

	}
	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();

	System.out.println("Clustering");
	clustering.printGroupNamesAndSizes();

	IObjectGrouping cl2 = kmeans.oneIter(clustering, dotSim);
	System.out.println("Cl2");
	cl2.printGroupNamesAndSizes();
	/*
	try{
	    kmeans.setProperty("Number of clusters", 2);
	}catch(PropertyNotFoundException pnfe){
	    // Do nothing
	    System.out.println("PropertyNotFoundException!");
	}
	TopDownSplitter tds = new TopDownSplitter();
	tds.setSimilarity(dotSim);
	IObjectGrouping clustering = tds.cluster(rowSet, kmeans);
	clustering.removeEmptyGroups();
	*/

	/*
	BisectingKMeans bkm = new BisectingKMeans(dotSim);
	IObjectGrouping clustering = bkm.cluster(rowSet);
	clustering.removeEmptyGroups();
	*/


    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
