package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


// java -cp .:classes/ -Xms1024m -Xmx1024m infomat.SimpleClusterer 




/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ infomat.JohanEvaluator</code>
 *
 * @version          080619
 * @author           Magnus Rosell
 *
 */
public class JohanEvaluator{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Path were the files can be found.
	String path = "/afs/nada.kth.se/home/f/u1ya3frf/Public/Johan/Aftonbladet/";

	// Load the matrix
	// These have texts as rows and words as columns
	String matrixFileName = path + "matrix.xml";
	File matrixFile = new File(matrixFileName);
	IMatrixIO iomatrix = new IMatrixIO();
	IMatrix matrix = iomatrix.load(matrixFile);
	matrix.getColumnIObjectSet().makeStartGrouping();
	matrix.getRowIObjectSet().makeStartGrouping();


	// Load a categorization
	String categorizationFileName = path + "categorization.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping originalCategorization = 
	    iobio.load(rows, matrix, categorizationFile);

	// Load a clustering
	String clusteringFileName = path + 
	    "clustering1213870053986.xml";
	File clusteringFile = new File(clusteringFileName);
	rows=true; // row clustering
	IObjectGrouping clustering = 
	    iobio.load(rows, matrix, clusteringFile);
	
	// Texts not in the clustering are removed from the categorization.
	IObjectGrouping categorization = 
	    new IObjectGrouping(originalCategorization);
	categorization.removeObjectsNotIn(clustering);
	

	// Result path
	String resultPath = "/afs/nada.kth.se/home/f/u1ya3frf/Public/Johan/results/";

	// The current time
	long time = System.currentTimeMillis();

	// Similarity
	// The same similarity as used in the clusterings.
	// It is needed for the evaluation.
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Write Text Result
	// This is if you want to look at the clusterings.
	/*
	String resultFileName = resultPath +
	    "clusteringResult."+time+".xml";
	File resFile = new File(resultFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(matrix, clustering, dotSim, resFile);
	*/

	// Evaluate and write the results
	String evaluationFileName = resultPath +
	    "clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, dotSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
