package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.algorithms.reduction.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.Iterator;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


/**
 *
 * NOT WORKING IN VERSION 080828
 *
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/  -Xms3024m -Xmx3024m infomat.NegationClusterer</code>
 *
 * @version          080605
 * @author           Magnus Rosell
 *
 */
public class NegationClusterer{

    private static String sweStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
    private static String engStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt";

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	//	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/jtteLitenKorpus.corpus.tokenFile.xml";
	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/ATot20.randomPart1.corpus.tokenFile.xml";
	File file = new File(tokenFile);
	TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	IMatrix textMatrix = tokenIO.load(file);

	// Stoplista
	String stoplistFileName = sweStoplistFileName;
	//	if(language==1) // engelska
	//	    stoplistFileName = engStoplistFileName;
	Stoplist sl = new Stoplist(stoplistFileName);
	boolean stopRow=false;
	sl.removeIObjectsInStoplist(textMatrix, stopRow);

	IObjectGrouping rowGrouping = 
	    textMatrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    textMatrix.getRowIObjectSet().makeStartGrouping();
	/*
	  rowGrouping.sort();
	  colGrouping.sort();
	*/

	// Filter the matrix
	// The example is to small

	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(textMatrix);


	LocationGrouper locationGrouper = new LocationGrouper();
	IObjectGrouping textCategorization = 
	    locationGrouper.cluster(textMatrix.getRowIObjectSet());
	
	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(textMatrix);

	IObjectSet textSet = textMatrix.getRowIObjectSet();

	// Similarity
	boolean row = true; 
	SparseDotSimilarity textDotSim = 
	    new SparseDotSimilarity(textMatrix, row);


	KMeans textKmeans = new KMeans(textDotSim);
	IObjectGrouping textClustering = textKmeans.cluster(textSet);
	textClustering.removeEmptyGroups();



	// Negation
	Vector<IVector<IVectorIndex>> newRepr = 
	    Negator.gramSchmidt(textClustering, textDotSim);

	// Reduction
	Reducer reducer = new Reducer();
	IObjectGrouping reducedTextClustering = 
	    new IObjectGrouping("Reduced "+textClustering.getName());
	Vector<IObjectGroup> ordered = textClustering.getGroups();
	for(int i=0; i<ordered.size(); i++){
	    IObjectGroup group = ordered.get(i);
	    IVector<IVectorIndex> repr = newRepr.get(i);
	    IObjectGroup redGroup = 
		reducer.reduceGroup(group, textDotSim, repr);
	    reducedTextClustering.add(redGroup);
	}

	// Vector Categorization
	IVectorCategorizer ivc = new IVectorCategorizer(textDotSim, newRepr);
	IObjectGrouping ivcGrouping = ivc.cluster(textSet);
			   

	// The current time
	long time = System.currentTimeMillis();
	
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/Negation/";

	// Write Text Result
	String textResultFileName = 
	    resultPath+"textClusteringResult."+time+".xml";
	File textResFile = new File(textResultFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(textMatrix, textClustering, textDotSim, textResFile);

	String reducedTextResultFileName = 
	    resultPath+"reducedTextClusteringResult."+time+".xml";
	File reducedTextResFile = new File(reducedTextResultFileName);
	gtio.save(textMatrix, reducedTextClustering, textDotSim, reducedTextResFile);

	String vecCatTextResultFileName = 
	    resultPath+"vecCatTextClusteringResult."+time+".xml";
	File vecCatTextResFile = new File(vecCatTextResultFileName);
	gtio.save(textMatrix, ivcGrouping, textDotSim, vecCatTextResFile);



	// Evaluate and write the results
	String textEvaluationFileName = 
	    resultPath+"textClusteringEvaluation."+time+".xml";
	Evaluation textEval = 
	    new Evaluation(textMatrix, textClustering, textCategorization, textDotSim, ""+time);
	calculateAndSaveEvaluation(textEval, textEvaluationFileName);
	
	String reducedTextEvaluationFileName = 
	    resultPath+"reducedTextClusteringEvaluation."+time+".xml";
	Evaluation reducedTextEval = 
	    new Evaluation(textMatrix, reducedTextClustering, textCategorization, textDotSim, ""+time);
	calculateAndSaveEvaluation(reducedTextEval, reducedTextEvaluationFileName);
	
	String vecCatTextEvaluationFileName = 
	    resultPath+"vecCatTextClusteringEvaluation."+time+".xml";
	Evaluation vecCatTextEval = 
	    new Evaluation(textMatrix, ivcGrouping, textCategorization, textDotSim, ""+time);
	calculateAndSaveEvaluation(vecCatTextEval, vecCatTextEvaluationFileName);
	

	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }


    private static void printMatrix(IMatrix matrix){
	
	IObjectSet rowSet = matrix.getRowIObjectSet();
	Vector<IObject> rios = rowSet.getOrdered();
	for(int i=0; i<rios.size(); i++){
	    HashSet set = 
		matrix.getColumnIMatrixCellsForRowObject(rios.elementAt(i));
	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		System.out.print(" "+mc.getLocalCount());
	    }
	    System.out.println();
	}

    }


}


class Negator{

    public static Vector<IVector<IVectorIndex>> 
	gramSchmidt(IObjectGrouping textClustering, 
		    SparseISimilarity similarity){

	Vector<IVector<IVectorIndex>> gramSchmidt = 
	    new Vector<IVector<IVectorIndex>>();
	Vector<IObjectGroup> ordered = textClustering.getGroups();
	for(int i=0; i<ordered.size(); i++){
	    IObjectGroup group = ordered.get(i);
	    IVector<IVectorIndex> vec = similarity.getGroupRepresentation(group); //group.getIVector();
	    gramSchmidt.add(vec);
	}

	return gramSchmidt(gramSchmidt);
    }

    public static Vector<IVector<IVectorIndex>> 
	gramSchmidt(Vector<IVector<IVectorIndex>> in){

	Vector<IVector<IVectorIndex>> gramSchmidt = 
	    new Vector<IVector<IVectorIndex>>();
	for(int i=0; i<in.size(); i++){
	    IVector<IVectorIndex> vec = in.get(i);
	    gramSchmidt.add(vec);
	}

	IVector<IVectorIndex> first = gramSchmidt.get(0);
	first.normalize();
	for(int i=1; i<gramSchmidt.size(); i++){
	    IVector<IVectorIndex> nu = gramSchmidt.get(i);
	    for(int j=i-1; j>=0; j--){
		IVector<IVectorIndex> prev = gramSchmidt.get(j);
		float dp = IVector.dotProduct(nu, prev);
		IVector<IVectorIndex> dpPrev = new IVector<IVectorIndex>(prev);
		dpPrev.multiply(dp);
		nu.subtract(dpPrev);
	    }
	    nu.normalize();
	}

	return gramSchmidt;
    }





}
