package infomat.vectorspace.weighting;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import java.lang.Math.*;
import java.util.*;

/**
 * A class for weighting of a IMatrix.
 *
 * Currently works in a specific direction! Considers rows to be objects
 * (texts) and columns to be features (words).
 *
 * @version          080807
 * @author           Magnus Rosell
 */
public class TFWeighter extends Weighter{

    public TFWeighter(){
	super();
    }

    public TFWeighter(mro.util.properties.Properties properties){
	super(properties);
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("TF Weighter Properties", CurrentClassGetter.getClassName());
		
	try{

	    // Local row weight
	    Property p = new Property("Local row weight", true, "Use local weight for elements in their row");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Use it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    // Normalization
	    p = new Property("Row normalization", true, "Wether the rows are normalized after the weighting");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public String getWeighterName(){
	return "TF-Weighter";
    }


    private int localRowWeight = 2; // Lazy

    /**
     * Weight the IMatrix (the whole matrix, 
     * which may be more than the current IObjectGroupings, the picture).
     */
    public void weight(IMatrix matrix){ 
	int localRowWeight = this.localRowWeight; //(Integer) properties.getPropertyValue("Local row weight");
	boolean anyLocalRowWeight=true;
	if(localRowWeight==0)
	    anyLocalRowWeight=false;

	//Local weight
	boolean row = true;
	IObjectGroup objectGroup = matrix.getRowIObjectSet();
	IObjectGroup representationGroup = matrix.getColumnIObjectSet();
	HashMap<IObject, HashMap<IObject, Float>> mapMap = localWeightInGroupByGroup(objectGroup, representationGroup, matrix, row, properties);

	//	System.out.println("******* weight");
	//	System.out.println("mapMap: "+mapMap.size());
	if(mapMap!=null){
	    Vector<IObject> rowObjects =
		matrix.getRowIObjectSet().getOrdered();
	    for(int i=0; i<rowObjects.size(); i++){
		IObject rowObj = rowObjects.elementAt(i);
		HashMap<IObject, Float> map = mapMap.get(rowObj);
		//		System.out.println("map: "+map.size());
		/*
		Iterator<IObject> iter = map.keySet().iterator();
		while(iter.hasNext()){
		    System.out.println(" - "+iter.next());
		}
		*/

		HashSet cells = 
		    matrix.getColumnIMatrixCellsForRowObject(rowObj);
		//		System.out.println("cells: "+cells.size());
		if(cells==null)
		    continue;
		Iterator cellIt = cells.iterator();
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    //		    System.out.println(mc.getRowIObject() +", "+mc.getColumnIObject()+": "+mc.getWeight());
		    //		    System.out.println("other: "+mc.getOtherIObject(rowObj));
		    float localWeight = map.get(mc.getOtherIObject(rowObj));
		    mc.setLocalWeight(localWeight);
		    mc.setWeight(localWeight);
		}
	    }
	}




	boolean rowNormalization = (Boolean) properties.getPropertyValue("Row normalization");
	if(rowNormalization){
	    row = true;
	    Normalizer.normalizeMatrix(matrix, row);
	}

    }



    /**
     * Returns the sum of the local weight as calculated in 
     * #localWeightInGroupByGroup.
     */
    public static HashMap<IObject, Float> totalLocalWeightInGroupByGroup(IObjectGroup objectGroup, IObjectGroup representationGroup, IMatrix matrix, boolean row, mro.util.properties.Properties properties){ 
	
	HashMap<IObject, Float> totalLocalWeightMap = new HashMap<IObject, Float>();

	HashMap<IObject, HashMap<IObject, Float>> mapMap = localWeightInGroupByGroup(objectGroup, representationGroup, matrix, row, properties);
	System.out.println("mapMap, size: "+mapMap.size());

	/*******/
	/*
	Iterator<Map.Entry<IObject, HashMap<IObject, Float>>> tempIt = 
	    mapMap.entrySet().iterator();
	while(tempIt.hasNext()){
	    Map.Entry<IObject, HashMap<IObject, Float>> entry = tempIt.next();
	    System.out.println("m: "+entry.getKey().getString());
	    HashMap<IObject, Float> tempMap = entry.getValue();
	    Iterator<Map.Entry<IObject, Float>> tempIt2 = 
		tempMap.entrySet().iterator();
	    while(tempIt2.hasNext()){
		Map.Entry<IObject, Float> entry2 = tempIt2.next();
		System.out.println(entry2.getKey().getString()+": "+entry2.getValue());
	    }
	}
	*/
	/*******/


	// Hr ngonstans. Antagligen vnt p rader och kolumner.

	Iterator<HashMap<IObject, Float>> it = mapMap.values().iterator();
	while(it.hasNext()){
	    HashMap<IObject, Float> weightMap = it.next();
	    //	    System.out.println("weightMap, size: "+weightMap.size());

	    Iterator<Map.Entry<IObject,Float>> it2 = 
		weightMap.entrySet().iterator();
	    while(it2.hasNext()){
		Map.Entry<IObject,Float> entry = it2.next();
		//		System.out.println(" -0 "+entry.getKey().getString()+", "+entry.getValue());
		Float sum = totalLocalWeightMap.get(entry.getKey());
		//		System.out.println(" -1 "+sum);
		if(sum!=null)
		    totalLocalWeightMap.put(entry.getKey(), 
					    sum+entry.getValue());
		else
		    totalLocalWeightMap.put(entry.getKey(), entry.getValue());
		//		System.out.println(" -2 "+totalLocalWeightMap.get(entry.getKey()));
	    }
	}
	return totalLocalWeightMap;
    }


    /**
     * The central local weighter. 
     * The elements of the first group are considered objects, and
     * those of the second their representation. The objects in the second
     * are weighted accoring to occurences for each object in the first.
     * For text-word: the words are weighted locally for each text.
     * OBS. Only works in row-col order yet!!!
     *
     * @param row Wether the first group is along the rows of the matrix.
     */
    public static HashMap<IObject, HashMap<IObject, Float>> localWeightInGroupByGroup(IObjectGroup objectGroup, IObjectGroup representationGroup, IMatrix matrix, boolean row, mro.util.properties.Properties properties){ 

	int localRowWeight = 2; //  Lazy (Integer) properties.getPropertyValue("Local row weight");
	boolean anyLocalRowWeight=true;
	if(localRowWeight==0)
	    anyLocalRowWeight=false;

	// Local row weight
	// localRowWeight==1: use current local weight
	if(anyLocalRowWeight && localRowWeight!=1){ 

	    HashMap<IObject, HashMap<IObject, Float>> ret = 
		new HashMap<IObject, HashMap<IObject, Float>>();

	    //
	    // Think of all as a document-term-matrix.
	    // Calculating the local weight for the terms (columns) 
	    // in each document (row).
	    //
	    Vector<IObject> rowObjects = objectGroup.getOrdered();
	    Vector<IObject> colObjects = representationGroup.getOrdered();
	
	    // OBS. The weighting is not completely general, yet.
	    double localWeightImportanceFactor = 0.0; // (Double) properties.getPropertyValue("(Local column/global row) weight importance factor");
	    /*
	    if(row){
		localWeightImportanceFactor = (Double) properties.getPropertyValue("(Local row/global column) weight importance factor");
	    }
	    */

	    for(int i=0; i<rowObjects.size(); i++){
		HashMap rowMap = new HashMap<IObject, Float>();

		IObject rowObj = rowObjects.elementAt(i);
		//	    Vector cells = matrix.getCellsWith(rowObj);
		Vector cells = matrix.getCellsWithIObjectRestrictedToGroup(rowObj, representationGroup);
		if(cells==null)
		    continue;
		Iterator cellIt = cells.iterator();
		float total = 0f;
		float maxCount = 0f;
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    total += mc.getLocalCount();
		    maxCount = (float) Math.max(maxCount, mc.getLocalCount());
		}
		cellIt = cells.iterator();
		while(cellIt.hasNext()){
		    IMatrixCell mc = (IMatrixCell) cellIt.next();
		    float localWeight=0.0f;
		    if(localRowWeight==2){ // simple relative frequency
			localWeight = mc.getLocalCount()/total;
		    }else if(localRowWeight==3){ // Croft 1983
			localWeight = (float)
			    (localWeightImportanceFactor + 
			     (1-localWeightImportanceFactor) * 
			     mc.getLocalCount()/maxCount);
		    }
		    rowMap.put(mc.getOtherIObject(rowObj), localWeight);
		    //		mc.setLocalWeight(localWeight);
		    //		mc.setWeight(localWeight);
		}		
		ret.put(rowObj, rowMap);

	    }
	    return ret;
	}
	return null;
    }


}
