package infomat.view;

import infomat.vectorspace.*;
import infomat.view.group.*;
import infomat.view.grouping.*;
import infomat.view.grouping.settings.*;
import infomat.view.info.*;
import infomat.view.menu.*;
import infomat.view.picture.*;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The panel that displays the Infomat Image.
 *
 * @version          061228
 * @author           Magnus Rosell
 */
public class InfomatView extends JPanel 
    implements Observer, ComponentListener {

    // The Image
    private SparsePicture sparsePicture = null;
    //Color backgroundColor=Color.WHITE;
    boolean columnGrid=false;
    boolean rowGrid=false;
    boolean gridChanged=false;
    private GridLayout gridLayout;
    boolean forceChange=false;

    // Grouping Settings
    //private ColoringManager coloringManager;
    private GroupingSettingsManager rowGroupingManager;
    private GroupingSettingsManager colGroupingManager;
    //private static float MIN_COLOR_COMPONENT_VALUE = 0.0f;

    private GroupFrameManager groupFrameManager;

    // The IMatrix
    // "From" is first element
    // "To" is the last plus one element
    // This should be consequent through the code.
    private int rowFrom, rowTo;
    private int colFrom, colTo;

    // The picture
    private int WIDTH;
    private int HEIGHT;
    private int oldWIDTH;
    private int oldHEIGHT;
	
    private float opacity = 0.5f;
    private float oldOpacity;
    private float minOpacity;
    private float oldMinOpacity;
    private float maxOpacity;
    private float oldMaxOpacity;
	
	
    // Dimension names, not object kind
    public final static String ROW_DIMENSION_NAME = "Row";
    public final static String COLUMN_DIMENSION_NAME = "Column";

    // The Model
    private InfomatModel model;

    // Mouse
    public final static int MOUSE_CHANGE_STATE_NONE = 0;
    public final static int MOUSE_CHANGE_STATE_DRAG = 1;
    public final static int MOUSE_CHANGE_STATE_ZOOMAREA = 2;
    public final static int MOUSE_CHANGE_STATE_ZOOM = 3;
    public final static int MOUSE_REMOVE_COLUMNS = 4;
    public final static int MOUSE_REMOVE_ROWS = 5;
    public final static int MOUSE_SELECT_PIXEL = 6;
	
    // private final static int MOUSESTATE_NONE = 0;
    private int mouseState;
    IObject dragStartColObject = null;
    IObject dragStartRowObject = null;

    // Zoom
    public final static double ZOOM_IN_FACTOR = 0.877; //0.123;
    public final static double ZOOM_OUT_FACTOR = 1.1; //0.1;


    private ToolbarView toolbarView;
    //private InfoPanel infoPanel;
	
    //    private PixelInfoFrame pixelInfoFrame;
	
    private InfomatOverview overview;

    //private JFrame overviewFrame;

    //private boolean changedGroupingOrder=false;


    public InfomatView(InfomatModel model) {
	this.model = model;
	model.setView(this);
	gridLayout = new GridLayout(1,1);
	setLayout(gridLayout);
	setPreferredSize(new Dimension(700, 70));
	// setSize(new Dimension(70,70));

	//coloringManager=new ColoringManager(this);
	boolean row = true;
	rowGroupingManager=new GroupingSettingsManager(this, model, row);
	row=false;
	colGroupingManager=new GroupingSettingsManager(this, model, row);

	groupFrameManager = new GroupFrameManager();

	WIDTH = HEIGHT = 0;

	addComponentListener(this);
	mouseState = MOUSE_CHANGE_STATE_NONE;

	overview = new InfomatOverview("Infomat - Overview", model, this, toolbarView);
	overview.setVisible(true); // Klickas fram frn menyn i starten

	//	pixelInfoFrame = new PixelInfoFrame("Pixel Info", model, this);
	//	pixelInfoFrame.setVisible(false);
		
    }

    public GroupFrameManager getGroupFrameManager(){
	return groupFrameManager;
    }

    public void setGridChanged(boolean grid){
	gridChanged=grid;
    }

    public boolean getGridChanged(){
	return gridChanged;
    }

    public boolean toggleColumnGrid(boolean toggle) {
	if (sparsePicture!=null) {
	    columnGrid=!columnGrid;
	    setGridChanged(true);
	    //overview.repaint();
	    model.update();
	}
	return columnGrid;
    }

    public boolean toggleRowGrid(boolean toggle) {
	if (toggle) {
	    rowGrid = !rowGrid;
	    setGridChanged(true);
	    //overview.repaint();
	    model.update();
	}
	return rowGrid;
    }

    public void toggleGuide() {
	if(sparsePicture!=null)
	    sparsePicture.getGuide().toggleDraw();
    }

    public void toggleOverview(){
	overview.setVisible(!overview.getVisible());
	if(overview.getVisible()){
	    //overview.forceChange();
	    overview.changePartPictures();
	    //overview.update(null, null);
	    model.update();
	}
    }
	
    /*
    public void togglePixelInfo(){
	pixelInfoFrame.setVisible(!pixelInfoFrame.getVisible());	
    }
    */
	
    /**
     * Save toolbarView for later use. Must be done!
     * 
     * @param toolbarView
     *            The toolbarView
     */
    public void setToolbarView(ToolbarView toolbarView) {
	this.toolbarView = toolbarView;
    }

    public ToolbarView getToolbarView(){
	return toolbarView;
    }

    /*
      public void setInfoPanel(InfoPanel infoPanel){
      this.infoPanel=infoPanel;
      }
	
      public InfoPanel getInfoPanel(){
      return infoPanel;
      }
    */
	
    /*
    public PixelInfoFrame getPixelInfoFrame(){
	return pixelInfoFrame;
    }
    */

    /**
     * Reset zoom.
     */
    public void resetImage() {
	Dimension d = model.getIMatrix().getCurrentSize();
	boolean change=false;
	if(rowFrom != 0 || rowTo != d.height ||
	   colFrom != 0 || colTo != d.width)
	    change=true;
	if(change){
	    rowFrom = 0;
	    rowTo = d.height;
	    colFrom = 0;
	    colTo = d.width;
	    forceChange();
	}
    }

    public void setMouseState(int state) {
	mouseState = state;

	if(state==MOUSE_REMOVE_ROWS)
	    System.out.println("state=MOUSE_REMOVE_ROWS");
	if(state==MOUSE_REMOVE_COLUMNS)
	    System.out.println("state=MOUSE_REMOVE_COLUMNS");
		
	/*
	  if(state==MOUSE_SELECT_PIXEL){
	  pixelInfoFrame.hold();
	  }else{
	  pixelInfoFrame.unHold();
	  }
	*/

    }



    /*
      public void writeInfomatIMatrixToPNGFile(File file){
      try{
      ImageIO.write(sparsePicture.getImage(), "png", file);
      }catch(IOException ioe){
      System.out.println("Filfel!");
      }
      }
    */



    public void paintComponent(Graphics g) {
	super.paintComponent(g);
    }

    private void printIMatrixBoundaries(){
	System.out.println("rowFrom: "+rowFrom+", rowTo: "+rowTo);
	System.out.println("colFrom: "+colFrom+", colTo: "+colTo);
    }


    /** Called by the Observable */
    public void update(Observable o, Object arg) {
	System.out.println("************ update - InfomatView");

	if (HEIGHT == 0 || WIDTH == 0 || !model.hasIMatrix()){
	    sparsePicture=null;
	    removeAll();
	}else if (sparsePicture == null){
	    Dimension d = model.getIMatrix().getCurrentSize();
	    rowFrom = 0;
	    rowTo = d.height;
	    colFrom = 0;
	    colTo = d.width;

	    changePartPictures();
	    overview.changePartPictures();
	    //overview.forceChange();
	}else if (sparsePicture != null){
	    if(parametersChanged()){ // || model.inquireFundamentalChanges()){
		System.out.println("parametersChanged: "+parametersChanged());
		changePartPictures();
		overview.changePartPictures();
		//overview.forceChange();
	    }else{
		boolean opacityChanged=false;
		if(opacityChanged()){
		    changeOpacity();
		    opacityChanged=true;
		}
		if(columnOpacityChanged()){
		    changeColumnOpacity();
		    opacityChanged=true;
		}
		if(opacityChanged){
		    sparsePicture.makeImage();
		    //overview.setOpacityChanged();
		    overview.makeImage();
		}
				
	    }
	    //	    }else if(alteredGroupingOrder){
	    //		sparsePicture.repaint();
	    //		alteredGroupingOrder=false;
	    //	    }
	}

	//overview.update(o, arg);
	overview.repaint();
	revalidate();
	repaint();
    }

    /*
      public void setAlteredGroupingOrder(boolean altered){
      alteredGroupingOrder=altered;
      }

      public boolean alteredGroupingOrder(){
      return alteredGroupingOrder;
      }
    */

    /*
      public float getMinColorComponentValue(){
      return MIN_COLOR_COMPONENT_VALUE;
      }
    */
	
    private boolean opacityChanged(){
	boolean ret=false;
	IObjectGrouping rowGrouping = model.getIMatrix().getCurrentRowGrouping();
	GroupingSettings gs = rowGroupingManager.getSettingsForGrouping(rowGrouping);		
	if(gs.opacityChanged())
	    ret=true;
	System.out.println("opacityChanged() - "+ret);
		
	return ret;
    }
	
    private void changeOpacity(){
	IObjectGrouping rowGrouping = model.getIMatrix().getCurrentRowGrouping();
	GroupingSettings gs = rowGroupingManager.getSettingsForGrouping(rowGrouping);		
	sparsePicture.adjustOpacity(gs.getMinOpacity(), gs.getMaxOpacity());
	SparsePicture ovSparsePicture = overview.getSparsePicture();
	if(ovSparsePicture!=null)
	    ovSparsePicture.adjustOpacity(gs.getMinOpacity(), gs.getMaxOpacity());
	gs.setOpacity();
    }

    private boolean columnOpacityChanged(){
	boolean ret=false;
		
	boolean row = false;
	GroupingSettingsManager gm = getGroupingManager(row);
	GroupingColoring colColoring = gm.getCurrentColoring();
	if(colColoring!=null){
	    IObjectGrouping colGrouping = colColoring.getIObjectGrouping();
	    GroupingSettings gs = gm.getSettingsForGrouping(colGrouping);		
	    if(gs.opacityChanged())
		ret=true;
	}
	/*
	// ej rtt - ska vara color grouping for column
	IObjectGrouping colGrouping = model.getIMatrix().getCurrentColumnGrouping();
	GroupingSettings gs = colGroupingManager.getSettingsForGrouping(colGrouping);		
	if(gs.opacityChanged())
	ret=true;
	*/
	return ret;
    }
	
    private void changeColumnOpacity(){
	boolean row = false;
	GroupingSettingsManager gm = getGroupingManager(row);
	GroupingColoring colColoring = gm.getCurrentColoring();
	if(colColoring!=null){
	    IObjectGrouping colGrouping = colColoring.getIObjectGrouping();
	    GroupingSettings gs = gm.getSettingsForGrouping(colGrouping);		
	    sparsePicture.adjustColumnColorOpacity(gs.getMinOpacity(), gs.getMaxOpacity());
	    SparsePicture ovSparsePicture = overview.getSparsePicture();
	    if(ovSparsePicture!=null)
		ovSparsePicture.adjustColumnColorOpacity(gs.getMinOpacity(), gs.getMaxOpacity());
	    gs.setOpacity();
	}
		
		
	/*
	  IObjectGrouping colGrouping = model.getIMatrix().getCurrentColumnGrouping();
	  GroupingSettings gs = colGroupingManager.getSettingsForGrouping(colGrouping);		
	  sparsePicture.adjustColumnColorOpacity(gs.getMinOpacity(), gs.getMaxOpacity());
	  gs.setOpacity();
	*/
    }

	
    protected void changePartPictures(){

	System.out.println("------------  changePartPictures");


	if(model==null || model.getIMatrix()==null){
	    removeAll();
	    return; 
	}

	if(sparsePicture!=null){
	    sparsePicture.removeAll();
	    sparsePicture=null;
	}



	/*
	  model.getIMatrix().getCurrentColumnGrouping().sort();
	  model.getIMatrix().getCurrentRowGrouping().sort();
	*/

	//	sparsePicture = 
	//	    model.getIMatrix().zoom(getSize(), xmin, xmax, ymin, ymax);
	//	sparsePicture = 
	//	    model.getIMatrix().zoom(getSize(), colFrom, colTo, rowFrom, rowTo);
		
	sparsePicture = 
	    SparsePictureConstructor.zoom(this, model.getIMatrix(), 
					  getSize(), 
					  colFrom, colTo, rowFrom, rowTo);
		
	if(sparsePicture==null){
	    removeAll();
	}else{

	    oldWIDTH=WIDTH;
	    oldHEIGHT=HEIGHT;
	    WIDTH = getSize().width;
	    HEIGHT = getSize().height;
		
	    sparsePicture.setView(this);
		
	    changeOpacity();
	    changeColumnOpacity();
		
	    sparsePicture.makeImage();

	    // Fljande fyra rader behvs
	    // d det ndras beroende p om det r flera delbilder.
	    // Se SparsePictureConstructor.
	    rowFrom=sparsePicture.getRowFrom();
	    rowTo=sparsePicture.getRowTo();
	    colFrom=sparsePicture.getColFrom();
	    colTo=sparsePicture.getColTo();

	    ////////////////////////////////////
	    //	    sparsePicture.printFirstPixelRowsRowObjects();


	    removeAll();
	    add(sparsePicture);
	    sparsePicture.setSize(getSize());
		
	    overview.forceChange();
	}
    }


    public void forceChange(){
	forceChange=true;
    }


    private boolean parametersChanged(){
	boolean ret=false;

	if(sparsePicture.getRowFrom() != rowFrom) ret=true;
	if(sparsePicture.getRowTo() != rowTo) ret=true;
	if(sparsePicture.getColFrom() != colFrom) ret=true;
	if(sparsePicture.getColTo() != colTo) ret=true;

		
	if(WIDTH!=oldWIDTH) ret=true;
	if(HEIGHT!=oldHEIGHT) ret=true;
	oldWIDTH=WIDTH;
	oldHEIGHT=HEIGHT;
		
	if(forceChange){
	    ret=true;
	    forceChange=false;
	}

	return ret;
    }



    /**
     * Set the opacity for the image.
     * 
     * @param opacity
     *            The opacity, a double between 0.0 and 1.0
     */
    public void setOpacity(float opacity) {
	this.opacity = opacity;
	//	sparsePicture.setOpacity(opacity);

	model.update();
    }

    public void setMinOpacity(float opacity) {
		
    }

    public void setMaxOpacity(float opacity) {
		
    }

	
	
    public void validateIMatrixBorders() {
	Dimension d = model.getIMatrix().getCurrentSize();

	if(colTo > d.width)
	    colTo=d.width;
	if(colFrom < 0)
	    colFrom = 0;

	if(rowTo > d.height)
	    rowTo=d.height;
	if(rowFrom < 0)
	    rowFrom = 0;
    }

    public int getRowFrom(){
	return rowFrom;
    }

    public int getRowTo(){
	return rowTo;
    }

    public int getColFrom(){
	return colFrom;
    }

    public int getColTo(){
	return colTo;
    }


    private double getZoomInFactor(){
	return InfomatView.ZOOM_IN_FACTOR;
    }

    private double getZoomOutFactor(){
	return InfomatView.ZOOM_OUT_FACTOR;
    }

    public void zoom(IObject colCenterObject, IObject rowCenterObject, 
		     int notches){
	if(sparsePicture==null)
	    return;
		
	System.out.println("---------------------------");
	System.out.println("zoom - InfomatView");
	System.out.println(colCenterObject.getString());
	System.out.println(rowCenterObject.getString());

	printIMatrixBoundaries();

	int colCenter = model.getIMatrix().getCurrentColumnGrouping().getIObjectOrderIndex(colCenterObject);
	int rowCenter = model.getIMatrix().getCurrentRowGrouping().getIObjectOrderIndex(rowCenterObject);

	System.out.println("rowCenter: "+rowCenter);
	System.out.println("colCenter: "+colCenter);


	int numOfCols = sparsePicture.getColTo() - sparsePicture.getColFrom();
	int numOfRows = sparsePicture.getRowTo() - sparsePicture.getRowFrom();

	double colChange = 0;
	double rowChange = 0;
	if (notches > 0) { // scrolled out
	    colChange = numOfCols * getZoomOutFactor();
	    rowChange = numOfRows * getZoomOutFactor();
	} else if (notches < 0) { // scrolled in
	    colChange = numOfCols * getZoomInFactor();
	    rowChange = numOfRows * getZoomInFactor();
	} else {
	    colChange = numOfCols;
	    rowChange = numOfRows;
	}

	int newColFrom = (int) Math.round(colCenter - colChange/2); 
	int newColTo = (int) Math.round(colCenter + colChange/2); 

	int newRowFrom = (int) Math.round(rowCenter - rowChange/2); 
	int newRowTo = (int) Math.round(rowCenter + rowChange/2); 

	if(newColTo-newColFrom<5 &&
	   newRowTo-newRowFrom<5)
	    return;
	else if(newColTo-newColFrom<5){
	    rowFrom=newRowFrom;
	    rowTo=newRowTo;		
	}else if(newRowTo-newRowFrom<5){
	    colFrom=newColFrom;
	    colTo=newColTo;
	}else{
	    colFrom=newColFrom;
	    colTo=newColTo;
	    rowFrom=newRowFrom;
	    rowTo=newRowTo;
	}
		
	//printIMatrixBoundaries();
	validateIMatrixBorders();
	//printIMatrixBoundaries();


	model.update();

    }

    public void fixMove(int newColFrom, int newColTo, 
			int newRowFrom, int newRowTo){

	Dimension d = model.getIMatrix().getCurrentSize();




	if(newColTo > d.width){
	    int diff = d.width-colTo;
	    colTo=d.width;
	    colFrom+= diff;
	}else if(newColFrom < 0){
	    int diff = 0-colFrom;
	    colFrom = 0;
	    colTo += diff;
	}else{
	    colTo=newColTo;
	    colFrom=newColFrom;
	}

	if(newRowTo > d.height){
	    int diff = d.height-rowTo;
	    rowTo=d.height;
	    rowFrom += diff;
	}else if(newRowFrom < 0){
	    int diff = 0-rowFrom;
	    rowFrom = 0;
	    rowTo += diff;
	}else{
	    rowTo=newRowTo;
	    rowFrom=newRowFrom;
	}
    }


    public void setDragStart(IObject colObject, IObject rowObject){
	dragStartColObject = colObject;
	dragStartRowObject = rowObject;
    }

    public void dragged(IObject colObject, IObject rowObject, boolean overviewPicture){
	if(dragStartColObject!=null && dragStartRowObject!=null){
	    if(colObject==null || rowObject==null){

		// Gammalt: Skriv info om att man bara kan "dra" inom kluster

		dragStartColObject=null;
		dragStartRowObject=null;
		return;
	    }

	    IMatrix m = model.getIMatrix();
	    int firstCol = m.getCurrentColumnGrouping().getIObjectOrderIndex(dragStartColObject);
	    int firstRow = m.getCurrentRowGrouping().getIObjectOrderIndex(dragStartRowObject);
	    int lastCol = m.getCurrentColumnGrouping().getIObjectOrderIndex(colObject);
	    int lastRow = m.getCurrentRowGrouping().getIObjectOrderIndex(rowObject);

	    int maxCol = Math.max(firstCol, lastCol); //+1;
	    int maxRow = Math.max(firstRow, lastRow); //+1;
	    int minCol = Math.min(firstCol, lastCol);
	    int minRow = Math.min(firstRow, lastRow);

	    dragStartColObject=null;
	    dragStartRowObject=null;

	    if (mouseState == MOUSE_CHANGE_STATE_DRAG) {
		System.out.println("... drag");


		int newColFrom = colFrom - (lastCol - firstCol);
		int newColTo = colTo - (lastCol - firstCol);
		int newRowFrom = rowFrom - (lastRow - firstRow);
		int newRowTo = rowTo - (lastRow - firstRow);
		if(overviewPicture){
		    newColFrom = colFrom + (lastCol - firstCol);
		    newColTo = colTo + (lastCol - firstCol);
		    newRowFrom = rowFrom + (lastRow - firstRow);
		    newRowTo = rowTo + (lastRow - firstRow);
		}
				
		fixMove(newColFrom, newColTo, newRowFrom, newRowTo);

		printIMatrixBoundaries();
		model.update();
	    }else if (mouseState == MOUSE_CHANGE_STATE_ZOOMAREA) {
		System.out.println("...till zoomarea ");

		if(maxCol-minCol<5 &&
		   maxRow-minRow<5)
		    return;
		else if(maxCol-minCol<5){
		    rowFrom = minRow;
		    rowTo = maxRow;
		}else if(maxRow-minRow<5){
		    colFrom = minCol;
		    colTo = maxCol;
		}else{			
		    colFrom = minCol;
		    colTo = maxCol;
		    rowFrom = minRow;
		    rowTo = maxRow;
		}
		System.out.println("Zoom to: ");
		printIMatrixBoundaries();

		//		validateIMatrixBorders();
		model.update();
	    }else if (mouseState == MOUSE_REMOVE_COLUMNS) {
		System.out.print("...till ta bort kolumner: ");
		System.out.println(minCol+", "+(maxCol));

		model.removeColumns(minCol, maxCol);
				
		IObjectGrouping grouping = model.getIMatrix().getCurrentColumnGrouping();
		getColumnGroupingManager().getColoringForGrouping(grouping).recolor();
		validateIMatrixBorders();
		forceChange();
		overview.forceChange();

		model.update();
	    } else if (mouseState == MOUSE_REMOVE_ROWS) {
		System.out.print("...till ta bort rader: ");
		System.out.println(minRow+", "+(maxRow));

		model.removeRows(minRow, maxRow);
				
		IObjectGrouping grouping = model.getIMatrix().getCurrentRowGrouping();
		getRowGroupingManager().getColoringForGrouping(grouping).recolor();
		validateIMatrixBorders();
		forceChange();
		overview.forceChange();
		model.update();
	    }
	}
    }



    public void componentHidden(ComponentEvent e) {
	//	sparsePicture.componentHidden(e);
    }

    public void componentMoved(ComponentEvent e) {
    }

    public void componentResized(ComponentEvent e) {
	Component c = e.getComponent();
	System.out.println("componentResized event from "
			   + c.getClass().getName() + "; new size: " + c.getSize().width
			   + ", " + c.getSize().height);
	oldWIDTH=WIDTH;
	oldHEIGHT=HEIGHT;
	WIDTH = c.getSize().width;
	HEIGHT = c.getSize().height;
	model.update();
    }

    public void componentShown(ComponentEvent e) {
	//	sparsePicture.componentShown(e);
    }


    /*
      protected float getMinOpacity() {
      return opacity;
      }

      protected float getMaxOpacity(){
      return 1f;
      }
    */
	
    /*
      protected Color getBackgroundColor(){
      return backgroundColor;
      }
    */

    public String getColumnDimensionName(){
	return COLUMN_DIMENSION_NAME;
    }

    public String getRowDimensionName(){
	return ROW_DIMENSION_NAME;
    }

    public boolean getRowGrid(){
	return rowGrid;
    }

    public boolean getColumnGrid(){
	return columnGrid;
    }

    public SparsePicture getSparsePicture(){
	return sparsePicture;
    }

    public int getMouseState(){
	return mouseState;
    }

    public InfomatOverview getOverview(){
	return overview;
    }
	
    /*
      public void disposeOverviewFrame() {
      overviewFrame.dispose();
      }
    */

    public void saveIMatrixPicture(File file){
	if(sparsePicture!=null)
	    sparsePicture.saveIMatrixPicture(file);
    }

    public GroupingSettingsManager getGroupingManager(boolean row){
	if(row)
	    return getRowGroupingManager();
	return getColumnGroupingManager();
    }
	
    public GroupingSettingsManager getColumnGroupingManager(){
	return colGroupingManager;
    }
    public GroupingSettingsManager getRowGroupingManager(){
	return rowGroupingManager;
    }

    public void setColumnGroupingManager(GroupingSettingsManager colGroupingManager){
	this.colGroupingManager=colGroupingManager;
    }

    public void setRowGroupingManager(GroupingSettingsManager rowGroupingManager){
	this.rowGroupingManager=rowGroupingManager;
    }
	
	
    public void invert(){
	int tmpColFrom = colFrom;
	int tmpColTo = colTo;
	colFrom=rowFrom;
	colTo=rowTo;
	rowFrom=tmpColFrom;
	rowTo=tmpColTo;
		
		
	GroupingSettingsManager tmp = rowGroupingManager;
	rowGroupingManager = colGroupingManager;
	colGroupingManager = tmp;
	colGroupingManager.invert(rowGroupingManager);

	getGroupFrameManager().invert();
    }
	
    public InfomatModel getInfomatModel(){
	return model;
    }
	
	
	
	
}
