package infomat.view.picture;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.imageio.*;
import javax.swing.*;

import infomat.vectorspace.*;
import infomat.view.*;
import infomat.view.info.*;
import infomat.view.menu.*;
import infomat.view.pixel.*;
import infomat.view.grouping.settings.GroupingColoring;


import mro.util.*;
import mro.util.doublehashmap.*;


/**
 * A SparsePicture is a projection of a IMatrix meant to be visualized
 * as a picture. It consists of one or several SparsePartPictures
 * which depicts part of the projection. Each SparsePartPictures 
 * is composed of many SparsePixels, the non zero pixels of the picture.
 */
public class SparsePicture extends JPanel
    implements MouseMotionListener, MouseListener{
    // MouseWheelListener

    // The View
    private InfomatView view=null;
    private ToolbarView toolbarView=null;
	
    // Extra JFrame for clicked pixel 
    private static PixelInfoFrame holdPixelInfoFrame;
	
    // Info
    //InfoPanel infoPanel=null;
	
    // Image
    private Dimension imageDimension;
    //private Vector<SparsePartPicture> partPictures = null;
    private SparsePartPicture[][] partPictureArray = null;
    private Color gridColor=Color.BLACK;
    private static int gridSize=1;
    private boolean overviewPicture=false;

    private DoubleHashMap coordToPixelMap;
	
    // The IMatrix
    private IMatrix matrix;
    private int rowFrom, rowTo;
    private int colFrom, colTo;

    private Guide guide;

    public SparsePicture(IMatrix matrix, Dimension imageDimension,
			 int colFrom, int colTo,
			 int rowFrom, int rowTo){ 
	// Image
	GridBagLayout layout = new GridBagLayout();
	setLayout(layout);
	//	setBackground(Color.BLACK);
	//	gridLayout = new GridLayout(1,1);
	//	setLayout(gridLayout);
	setPreferredSize(imageDimension); //new Dimension(700, 70));

	// Listeners
	addMouseMotionListener(this);
	//	addMouseWheelListener(this);
	addMouseListener(this);

	// Fields
	this.matrix = matrix;
	this.imageDimension = imageDimension;
	this.rowFrom=rowFrom;
	this.rowTo=rowTo;
	this.colFrom=colFrom;
	this.colTo=colTo;
	//partPictures = new Vector<SparsePartPicture>(); 

	guide = new Guide(this);
    }



    public void setView(InfomatView view){
	this.view=view;
    }

    public void setToolbarView(ToolbarView toolbarView){
	this.toolbarView=toolbarView;
    }

    /*
      public void setInfoPanel(InfoPanel infoPanel){
      this.infoPanel=infoPanel;
      }
    */
	
    public void addPartPictureArray(SparsePartPicture[][] arr){
	partPictureArray = arr;
    }

    public void makeCoordToPixelMap(){
	coordToPixelMap = new DoubleHashMap();
	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[cgi][rgi];
		Vector<SparsePixel> pixels = spp.getPixels();
		for(int i=0; i<pixels.size(); i++){
		    SparsePixel sp = pixels.elementAt(i);
		    int x = sp.getX() + spp.getOrigoX();
		    int y = sp.getY() + spp.getOrigoY();
		    coordToPixelMap.put(x, y, sp);
		}
				
	    }
	}

		
	/*
	  coordToPixelMap = new DoubleHashMap();
	  Vector<SparsePixel> pixels = getAllPixelsInPartPictures();
	  for(int i=0; i<pixels.size(); i++){
	  SparsePixel sp = pixels.elementAt(i);
	  int x = sp.getX();
	  int y = sp.getY();
	  Vector<IMatrixCell> cells = sp.getAllIMatrixCells();
	  for(int j=0; j<cells.size(); j++){
	  IMatrixCell mc = cells.elementAt(j);
	  //coordToPixelMap.put(mc.getRowIObject(), mc.getColumnIObject(), sp);
	  coordToPixelMap.put(x, y, sp);
	  }
	  }				 
	*/
	
    }
	
    public SparsePixel getPixelForObjects(IObject rowObject, IObject colObject){
	//return (SparsePixel) coordToPixelMap.get(rowObject, colObject);
	int c = matrix.getCurrentColumnGrouping().getIObjectOrderIndex(colObject);
	int x = getClosestXPixelForColumn(c);
	int r = matrix.getCurrentRowGrouping().getIObjectOrderIndex(rowObject);
	int y = getClosestYPixelForRow(r);
	return (SparsePixel) coordToPixelMap.get(x, y);
    }
	

    /*
      public void addPartPicture(SparsePartPicture spp){
      partPictures.add(spp);
      spp.setSparsePicture(this);
      }
    */

    public void setOverviewPicture(boolean overviewPicture){
	this.overviewPicture = overviewPicture;
    }

    public boolean getOverviewPicture(){
	return overviewPicture;
    }

    private Color getZoomRectColor(){
	return InfomatOverview.ZOOMRECT_COLOR;
    }

    public int getRowFrom(){
	return rowFrom;
    }

    public int getRowTo(){
	return rowTo;
    }

    public int getColFrom(){
	return colFrom;
    }

    public int getColTo(){
	return colTo;
    }


    public InfomatView getView(){
	return view;
    }

    public Color getGridColor(){
	return gridColor;
    }

    public static int getGridSize(){
	return gridSize;
    }

    public IObjectGroup getRowGroup(){
	return matrix.getCurrentRowGrouping();
    }

    public IObjectGroup getColumnGroup(){
	return matrix.getCurrentColumnGrouping();
    }


    public void makeImage(){
	System.out.println("SparsePicture - makeImage()");
	
	if(partPictureArray!=null){
	    for(int cgi=0; cgi<partPictureArray.length; cgi++){
		for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		    SparsePartPicture spp = partPictureArray[cgi][rgi];
		    spp.makeImage();
		}
	    }
	}
	
	/*
	  if(partPictures!=null && partPictures.size()>0){
	  //if(partPictures.size()>0){
	  if(partPictures.elementAt(0).getImage() == null){
	  for(int i=0; i<partPictures.size(); i++){
	  SparsePartPicture spp = partPictures.elementAt(i);
	  spp.makeImage();
	  }
	  }
	  }
	*/
    }


    private float[] findMinAndMaxInPixelVectorPart(Vector<SparsePixel> pixels){
	float[] minMax = new float[2];
	minMax[0]=Float.MAX_VALUE;
	minMax[1]=0.0f;
	int maxI=1000;
	
	/*
	  if(pixels.size()<2*maxI){
			
	  }else{
	*/
	for(int i=0; i<maxI; i++){
	    int r = (int) (pixels.size()* Math.random());
	    SparsePixel sp = (SparsePixel) pixels.elementAt(r);
	    float weight = sp.getWeight();
	    if(weight > minMax[1])
		minMax[1] = weight;
	    if(weight < minMax[0])
		minMax[0] = weight;
	}
	//		}
	return minMax;
    }

    /**
     * Change the distribution to be more even over 0.0 .. 1.0
     */
    public void adjustOpacity(float newMinWeight, float newMaxWeight){

	// Analyze a sample of the SparsePixel:s
	Vector<SparsePixel> pixels = getAllPixelsInPartPictures();
	float[] minMax = findMinAndMaxInPixelVectorPart(pixels);
	float currentMinWeight = minMax[0];
	float currentMaxWeight = minMax[1];
				
	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[cgi][rgi];
		spp.adjustOpacity(newMinWeight, newMaxWeight, 
				  currentMinWeight, 
				  currentMaxWeight);
	    }
	}

	/*
	  for(int i=0; i<partPictures.size(); i++)
	  partPictures.elementAt(i).adjustOpacity(newMinWeight, newMaxWeight, 
	  currentMinWeight, 
	  currentMaxWeight);
	*/
    }

    public void adjustColumnColorOpacity(float newMinWeight, float newMaxWeight){
	Vector<SparsePixel> pixels = getAllColumnColorPixelsInPartPictures();
	if(pixels.size()>0){
	    float[] minMax = findMinAndMaxInPixelVectorPart(pixels);
	    float currentMinWeight = minMax[0];
	    float currentMaxWeight = minMax[1];
	
	    for(int cgi=0; cgi<partPictureArray.length; cgi++){
		for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		    SparsePartPicture spp = partPictureArray[cgi][rgi];
		    spp.adjustColumnColorOpacity(newMinWeight, newMaxWeight, 
						 currentMinWeight, 
						 currentMaxWeight);
		}
	    }

	    /*
	      for(int i=0; i<partPictures.size(); i++)
	      partPictures.elementAt(i).adjustColumnColorOpacity(newMinWeight, newMaxWeight, 
	      currentMinWeight, 
	      currentMaxWeight);
	    */
	}
    }	
	
    public void makeAverageColumnColor(){
	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[cgi][rgi];
		spp.makeAverageColumnColor();
	    }
	}

	/*
	  for(int cgi=0; cgi<partPictureArray.length; cgi++){
	  for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
	  SparsePartPicture spp = partPictureArray[cgi][rgi];
	  spp.makeAverageColumnColor();
	  }
	  }
	*/
    }
	
	
    public void makeAverageColor(){//float minColor){

	// Analyze a sample of the SparsePixel:s
	//float currentMaxColor=0.0f;
	//float currentMinColor=Float.MAX_VALUE;

	Vector<SparsePixel> pixels = getAllPixelsInPartPictures(); 
	for(int i=0; i<pixels.size(); i++){
	    SparsePixel sp = (SparsePixel) pixels.elementAt(i);
	    sp.makeAverageColor();
	}

	/*
	  Vector<SparsePixel> pixels = getAllPixelsInPartPictures(); 
	  for(int i=0; i<pixels.size() && i<1000; i++){
	  SparsePixel sp = (SparsePixel) pixels.elementAt(i);
	  float[] rgb = sp.getRGB();
	  for(int c=0; c<3; c++){
	  if(rgb[c] > currentMaxColor)
	  currentMaxColor = rgb[c];
	  if(rgb[c] < currentMinColor)
	  currentMinColor = rgb[c];
	  }
	  sp.printColorParamters();
	  }

	  System.out.println("currentMinColor: "+currentMinColor);
	  System.out.println("currentMaxColor: "+currentMaxColor);

	  float newMinColor=minColor;
	  float newMaxColor=1.0f;
	  if(currentMinColor==currentMaxColor)
	  currentMinColor=0.0f;
	  for(int i=0; i<partPictures.size(); i++)
	  partPictures.elementAt(i).adjustColor(newMinColor, newMaxColor, 
	  currentMinColor, 
	  currentMaxColor);
	*/
    }


    public Vector<SparsePixel> getAllColumnColorPixelsInPartPictures(){
	Vector<SparsePixel> pixels = new Vector<SparsePixel>();
		
	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[cgi][rgi];
		Vector<SparsePixel> partPixels = 
		    spp.getColumnColorPixels();
		pixels.addAll(partPixels);
	    }
	}

	/*
	  if(partPictures!=null){
	  for(int i=0; i<partPictures.size(); i++){
	  SparsePartPicture partPic = partPictures.elementAt(i);
	  Vector<SparsePixel> partPixels = 
	  partPic.getColumnColorPixels();
	  pixels.addAll(partPixels);
	  }
	  }
	*/
	return pixels;
    }

    public Vector<SparsePixel> getAllPixelsInPartPictures(){
	Vector<SparsePixel> pixels = null; // new Vector<SparsePixel>();
	if(partPictureArray!= null){
	    pixels = new Vector<SparsePixel>();
		
	    for(int cgi=0; cgi<partPictureArray.length; cgi++){
		for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		    SparsePartPicture spp = partPictureArray[cgi][rgi];
		    Vector<SparsePixel> partPixels = 
			spp.getPixels();
		    pixels.addAll(partPixels);
		}
	    }
	}
	/*
	  if(partPictures!=null){
	  pixels = new Vector<SparsePixel>();
	  for(int i=0; i<partPictures.size(); i++){
	  SparsePartPicture partPic = partPictures.elementAt(i);

	  Vector<SparsePixel> partPixels = partPic.getPixels();
	  pixels.addAll(partPixels);
	  }
	  }
	*/
	return pixels;
    }


    private IObject getClosestRowIObject(int y){
	/*
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(y>=0 && y<spp.getImageHeight())
		return spp.getClosestRowIObject(y);
	}
	*/
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    int minY=spp.getOrigoY();
	    if(!view.getRowGrid()){
		minY += spp.getOffsetY();
	    }
	    if(y>=minY && y < minY+spp.getImageHeight()) //maxY)
		return spp.getClosestRowIObject(y-minY);
	}

	for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
	    SparsePartPicture spp = partPictureArray[0][rgi];
	    int minY=spp.getOrigoY();
	    if(!view.getRowGrid()){
		minY += spp.getOffsetY();
	    }
	    if(y>=minY && y < minY+spp.getImageHeight()) //maxY)
		return spp.getClosestRowIObject(y-minY);
	}
	return null;
    }



    private IObject getLastRowIObject(int y){
	/*
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(y>=0 && y<spp.getImageHeight())
		return spp.getLastRowIObject(y);
	}
	*/
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    int minY=spp.getOrigoY();
	    if(!view.getRowGrid()){
		minY += spp.getOffsetY();
	    }
	    if(y>=minY && y < minY+spp.getImageHeight()) //maxY)
		return spp.getLastRowIObject(y-minY);
	}


	for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
	    SparsePartPicture spp = partPictureArray[0][rgi];
	    int minY=spp.getOrigoY();
	    if(!view.getRowGrid()){
		minY += spp.getOffsetY();
	    }
	    if(y>=minY && y < minY+spp.getImageHeight()) //maxY)
		return spp.getLastRowIObject(y-minY);
	}
	return null;
    }
	
    /*
    public void printFirstPixelRowsRowObjects(){
	HashMap<IObject> rowObs = new HashMap<IObject>();
	
    }
    */





    private IObject getLastColumnIObject(int x){
	if(partPictureArray.length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(x>=0 && x<spp.getImageWidth())
		return spp.getLastColumnIObject(x);
	}

	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    SparsePartPicture spp = partPictureArray[cgi][0];
	    int minX=spp.getOrigoX();
	    if(!view.getColumnGrid()){
		minX += spp.getOffsetX();
	    }
	    if(x>=minX && x < minX+spp.getImageWidth())
		return spp.getLastColumnIObject(x-minX);
	}
	return null;
    }

	
    private IObject getClosestColumnIObject(int x){
	if(partPictureArray.length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(x>=0 && x<spp.getImageWidth())
		return spp.getClosestColumnIObject(x);
	}

	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    SparsePartPicture spp = partPictureArray[cgi][0];
	    int minX=spp.getOrigoX();
	    if(!view.getColumnGrid()){
		minX += spp.getOffsetX();
	    }
	    if(x>=minX && x < minX+spp.getImageWidth())
		return spp.getClosestColumnIObject(x-minX);
	}
	return null;
    }

	
	
	
    private int getClosestXPixelForColumn(int col){
	if(partPictureArray.length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(col>=spp.getColFrom() && col<spp.getColTo()){
		int minX = spp.getOrigoX();
		if(!view.getColumnGrid()){
		    minX += spp.getOffsetX();
		}
		return minX + 
		    spp.getClosestXPixelForColumn(col-spp.getColFrom());
	    }
	}

	for(int cgi=0; cgi<partPictureArray.length; cgi++){
	    SparsePartPicture spp = partPictureArray[cgi][0];
	    if(col>=spp.getColFrom() && col<spp.getColTo()){
		int minX = spp.getOrigoX();
		if(!view.getColumnGrid()){
		    minX += spp.getOffsetX();
		}
		return minX +
		    spp.getClosestXPixelForColumn(col-spp.getColFrom());
	    }
	}
	return -1;
    }

    private int getClosestYPixelForRow(int row){
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(row>=spp.getRowFrom() && row<spp.getRowTo()){
		int minY = spp.getOrigoY();
		if(!view.getRowGrid()){
		    minY += spp.getOffsetY();
		}
		return minY +
		    spp.getClosestYPixelForRow(row-spp.getRowFrom());
	    }
	}

	for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
	    SparsePartPicture spp = partPictureArray[0][rgi];
	    if(row>=spp.getRowFrom() && row<spp.getRowTo()){
		int minY = spp.getOrigoY();
		if(!view.getRowGrid()){
		    minY += spp.getOffsetY();
		}
		return minY +
		    spp.getClosestYPixelForRow(row-spp.getRowFrom());
	    }
	}

	return -1;
    }

    private SparsePixel getPixel(int x, int y){
	//	System.out.println("-------------");
	//	System.out.println(x+", "+y);

	int sppX = 0;
	int localX=-1;
	if(partPictureArray.length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(x>=0 && x<spp.getImageWidth())
		localX=x;
	}else{
	    for(int cgi=0; cgi<partPictureArray.length; cgi++){
		SparsePartPicture spp = partPictureArray[cgi][0];
		int minX=spp.getOrigoX();
		if(!view.getColumnGrid()){
		    minX += spp.getOffsetX();
		}
		if(x>=minX && x < minX+spp.getImageWidth()){
		    localX = x - minX;
		    sppX = cgi;
		    break;
		}
	    }
	}
	if(localX==-1)
	    return null;

	int sppY=0;
	int localY=-1;
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(y>=0 && y<spp.getImageHeight())
		localY=y;
	}else{

	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[0][rgi];
		int minY=spp.getOrigoY();
		if(!view.getRowGrid()){
		    minY += spp.getOffsetY();
		}
		if(y>=minY && y < minY+spp.getImageHeight()){
		    localY=y-minY;
		    sppY = rgi;
		    break;
		}
	    }
	}

	if(localY==-1)
	    return null;

	SparsePartPicture spp = partPictureArray[sppX][sppY];

	return spp.getPixel(localX, localY);
    }



    private IObjectGroup getColumnIObjects(int x){
	IObject startO = getClosestColumnIObject(x);
	IObject endO = getLastColumnIObject(x);
	/*
	if(startO==endO){
	    IObjectGroup ret = new IObjectGroup("Part of"+matrix.getCurrentColumnGrouping().getString());
	    ret.add(startO);
	    return ret;
	}
	*/
  
	return matrix.getCurrentColumnGrouping().subGroup(startO, endO);

	/*
	int sppX = 0;
	int localX=-1;
	if(partPictureArray.length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(x>=0 && x<spp.getImageWidth())
		localX=x;
	}else{
	    for(int cgi=0; cgi<partPictureArray.length; cgi++){
		SparsePartPicture spp = partPictureArray[cgi][0];
		int minX=spp.getOrigoX();
		if(!view.getColumnGrid()){
		    minX += spp.getOffsetX();
		}
		if(x>=minX && x < minX+spp.getImageWidth()){
		    localX = x - minX;
		    sppX = cgi;
		    break;
		}
	    }
	}
	if(localX==-1)
	    return null;

	SparsePartPicture spp = partPictureArray[sppX][0];
	return spp.getColumnIObjects(localX);
	*/
    }

    private IObjectGroup getRowIObjects(int y){
	IObject startO = getClosestRowIObject(y);
	IObject endO = getLastRowIObject(y);

	/*
	if(startO==endO){
	    IObjectGroup ret = new IObjectGroup("Part of"+matrix.getCurrentRowGrouping().getString());
	    ret.add(startO);
	    return ret;
	}
	*/

	return matrix.getCurrentRowGrouping().subGroup(startO, endO);

	/*
	int sppY=0;
	int localY=-1;
	if(partPictureArray[0].length==1){
	    SparsePartPicture spp = partPictureArray[0][0];
	    if(y>=0 && y<spp.getImageHeight())
		localY=y;
	}else{

	    for(int rgi=0; rgi<partPictureArray[0].length; rgi++){
		SparsePartPicture spp = partPictureArray[0][rgi];
		int minY=spp.getOrigoY();
		if(!view.getRowGrid()){
		    minY += spp.getOffsetY();
		}
		if(y>=minY && y < minY+spp.getImageHeight()){
		    localY=y-minY;
		    sppY = rgi;
		    break;
		}
	    }
	}

	if(localY==-1)
	    return null;

	SparsePartPicture spp = partPictureArray[0][sppY];
	return spp.getRowIObjects(localY);
	*/
    }



    protected SparsePartPicture[][] getPartPictureArray(){
	return partPictureArray;
    }

    protected int getColumnPartPictureNumberForColumnIndex(int ci){
		
	// Hr kanske man borde utg frn gruppering istllet fr bild...
	int cgi=0;
	for(cgi=0; cgi<partPictureArray.length; cgi++){
	    SparsePartPicture spp = partPictureArray[cgi][0];
	    if(ci>=spp.getColFrom() && ci<spp.getColTo())
		return cgi;
	}
	return -1;
    }
	
	
    public void mouseDragged(MouseEvent e) {
	mouseMoved(e);
    }

    public void mouseMoved(MouseEvent e) {
	if(toolbarView==null)
	    toolbarView=getView().getToolbarView();
		
	int x = e.getX();
	int y = e.getY();

	String rowInfo = "";
	IObject rowO = getClosestRowIObject(y);
	if(rowO!=null)
	    rowInfo=rowO.getString()+", "+matrix.getCurrentRowGrouping().getIObjectOrderIndex(rowO);

	String colInfo = "";
	IObject colO = getClosestColumnIObject(x);
	if(colO!=null)
	    colInfo=colO.getString()+", "+matrix.getCurrentColumnGrouping().getIObjectOrderIndex(colO);

	toolbarView.setInfo(""+colInfo, ""+rowInfo);
	//view.getInfoPanel().setCurrentInfo(rowO, colO);
	//	view.getPixelInfoFrame().setCurrentInfo(rowO, colO);

	setInfoPixelFrame(x, y);
	
	guide.draw(x, y);

	//	System.out.println("x: "+x+", y: "+y);
    }


    private void setInfoPixelFrame(int x, int y){
	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame==null)
	    return;

	//	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_CELLS){
	    SparsePixel sp = getPixel(x,y);
	    pixelFrame.setCurrentInfo(sp);
	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_ROWS){
	    boolean row = true;
	    IObjectGroup group = getRowIObjects(y);
	    //	    System.out.println("row group: "+group.numberOfIObjects());
	    pixelFrame.setCurrentInfo(group, row);
	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_COLUMNS){
	    boolean row = false;
	    IObjectGroup group = getColumnIObjects(x);
	    pixelFrame.setCurrentInfo(group, row);
	}
    }

    private void setSelectPixelFrame(int x, int y){
	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame==null)
	    return;

	//	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_CELLS){
	    SparsePixel sp = getPixel(x,y);
	    pixelFrame.selectCurrent(sp);
	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_ROWS){
	    boolean row = true;
	    IObjectGroup group = getRowIObjects(y);
	    pixelFrame.selectCurrent(group, row);
	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_COLUMNS){
	    boolean row = false;
	    IObjectGroup colGroup = getColumnIObjects(x);

	    System.out.println("setSelectPixelFrame");
	    System.out.println("--- colGroup: "+colGroup.numberOfIObjects());

	    pixelFrame.selectCurrent(colGroup, row);
	}
    }

    private void setSelectAreaPixelFrame(IObject lastRowObject, IObject lastColObject){
	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame==null)
	    return;

	//	PixelFrame pixelFrame = view.getInfomatModel().getPixelFrame();
	if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_CELLS){
	    IObjectGroup rowGroup = matrix.getRowIObjectSet().getCurrentGrouping().subGroup(dragStartRowObject, lastRowObject);
	    IObjectGroup colGroup = matrix.getColumnIObjectSet().getCurrentGrouping().subGroup(dragStartColObject, lastColObject);
	    
	    
	    Vector vec = matrix.subMatrix(rowGroup, colGroup).getAllCells();
	    pixelFrame.select(vec);


	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_ROWS){
	    boolean row = true;
	    IObjectGroup rowGroup = matrix.getRowIObjectSet().getCurrentGrouping().subGroup(dragStartRowObject, lastRowObject);

	    pixelFrame.selectCurrent(rowGroup, row);

	}else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_COLUMNS){
	    boolean row = false;
	    IObjectGroup colGroup = matrix.getColumnIObjectSet().getCurrentGrouping().subGroup(dragStartColObject, lastColObject);

	    /*
	    System.out.println("setSelectAreaPixelFrame");
	    System.out.println("--- colGroup: "+colGroup.numberOfIObjects());
	    */

	    pixelFrame.selectCurrent(colGroup, row);

	}
    }	

    


    public void mouseClicked(MouseEvent e) {
	/*
	System.out.println("Pressed button " + e.getButton()
			   + ", mouseState = " + getView().getMouseState());
	if (getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_ZOOM) {
	    if (e.getButton() == 1) {
		System.out.println("Translate to scroll -1");
		mouseWheelMoved(new MouseWheelEvent(this, 0, 0, 0, e.getX(), e
						    .getY(), 0, false, 0, 0, -1));
	    } else if (e.getButton() == 3) {
		System.out.println("Translate to scroll 1");
		mouseWheelMoved(new MouseWheelEvent(this, 0, 0, 0, e.getX(), e
						    .getY(), 0, false, 0, 0, 1));
	    }
	}
	*/
    }


    /*
    public void mousePressed(MouseEvent e) {
	if (getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_DRAG
	    || getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_ZOOMAREA
	    || getView().getMouseState() == InfomatView.MOUSE_REMOVE_COLUMNS
	    || getView().getMouseState() == InfomatView.MOUSE_REMOVE_ROWS
	    || getView().getMouseState() == InfomatView.MOUSE_SELECT_PIXEL) {
	    IObject colObject = getClosestColumnIObject(e.getX());
	    IObject rowObject = getClosestRowIObject(e.getY());

	    //	    System.out.println("Drag frn " + e + "...");
	    //System.out.println("Drag ...");
	    //System.out.println("x: "+e.getX()+", y: "+e.getY());

	    if(getView().getMouseState() == InfomatView.MOUSE_SELECT_PIXEL){
		//getView().getPixelInfoFrame().setHoldPixel(rowObject, colObject);
		if(holdPixelInfoFrame!=null)
		    holdPixelInfoFrame.dispose();
				
		String title = "Selected Pixel";
		holdPixelInfoFrame = 
		    new PixelInfoFrame(title, view.getInfomatModel(), view);
		holdPixelInfoFrame.setVisible(true);
		holdPixelInfoFrame.hold();
		holdPixelInfoFrame.setHoldPixel(rowObject, colObject);
		holdPixelInfoFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		//holdPixelInfoFrame.setShowAllPixels();
				
		//holdPixelInfoFrame.repaint();
	    }else{
		guide.setRect(e.getX(), e.getY());
		getView().setDragStart(colObject, rowObject);
	    }
	}
		
    }
    */

    public void mousePressed(MouseEvent e) {
	if (getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_DRAG
	    || getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_ZOOMAREA){
	    IObject colObject = getClosestColumnIObject(e.getX());
	    IObject rowObject = getClosestRowIObject(e.getY());


	    guide.setRect(e.getX(), e.getY());
	    getView().setDragStart(colObject, rowObject);

	}else if(getView().getMouseState() == InfomatView.MOUSE_REMOVE_ROWS){

	    IObject colObject = getClosestColumnIObject(e.getX());
	    IObject rowObject = getClosestRowIObject(e.getY());

	    getView().setDragStart(colObject, rowObject);
	    guide.setRows(e.getX(), e.getY());

	}else if(getView().getMouseState() == InfomatView.MOUSE_REMOVE_COLUMNS){
	    IObject colObject = getClosestColumnIObject(e.getX());
	    IObject rowObject = getClosestRowIObject(e.getY());

	    getView().setDragStart(colObject, rowObject);
	    guide.setCols(e.getX(), e.getY());

	}else if(getView().getMouseState() == InfomatView.MOUSE_SELECT_PIXEL){

	    PixelFrame pixelFrame = getView().getInfomatModel().getPixelFrame();
	    if(pixelFrame!=null){

		/*
		int x = e.getX();
		int y = e.getY();
		SparsePixel sp = getPixel(x,y);
		pixelFrame.setCurrentInfo(sp);
		*/
		int x = e.getX();
		int y = e.getY();
		setInfoPixelFrame(x,y);
		setSelectPixelFrame(x,y);

		int selectionState = pixelFrame.getSelectionState();
		if(selectionState == PixelFrame.SELECTION_STATE_SINGLE){
		    //		    pixelFrame.selectCurrent(sp);
		    
		}else if(selectionState == PixelFrame.SELECTION_STATE_AREA){
		    //		    pixelFrame.selectCurrent(sp);

		    IObject colObject = getClosestColumnIObject(e.getX());
		    IObject rowObject = getClosestRowIObject(e.getY());
		    setDragStart(colObject, rowObject);

		    if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_CELLS){

			guide.setRect(e.getX(), e.getY());

		    }else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_ROWS){
			guide.setRows(e.getX(), e.getY());
			
		    }else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_COLUMNS){
			guide.setCols(e.getX(), e.getY());
			
		    }




		}
	    }


	}
		
    }


    IObject dragStartColObject;
    IObject dragStartRowObject;


    private void setDragStart(IObject colObject, IObject rowObject){
	dragStartColObject = colObject;
	dragStartRowObject = rowObject;
    }



    public void mouseReleased(MouseEvent e) {

	System.out.println("mouseReleased()");
	System.out.println("x: "+e.getX()+", y: "+e.getY());

	IObject colObject = getLastColumnIObject(e.getX());
	IObject rowObject = getLastRowIObject(e.getY());
	/*
	  IObject colObject = getClosestColumnIObject(e.getX());
	  IObject rowObject = getClosestRowIObject(e.getY());
	*/
		
	/*
	  int x = e.getX() + 1;
	  IObject nextColObject = getClosestColumnIObject(x);
	  if(nextColObject==null){ // Kan vara en separator
	  x++;
	  nextColObject = getClosestColumnIObject(x);
	  }
	  if(nextColObject==null){
	  x = e.getX();
	  nextColObject = getClosestColumnIObject(x);
	  }
	  int col = matrix.getCurrentColumnGrouping().getIObjectOrderIndex(nextColObject);
	  if(x>e.getX())
	  col--;
	  IObject colObject = matrix.getCurrentColumnGrouping().getIObjectAt(col);

		
	  int y = e.getY() + 1;
	  IObject nextRowObject = getClosestRowIObject(y);
	  if(nextRowObject==null){ // Kan vara en separator
	  y++;
	  nextRowObject = getClosestRowIObject(y);
	  }
	  if(nextRowObject==null){
	  y = e.getY();
	  nextRowObject = getClosestRowIObject(y);
	  }
		
	  int row = matrix.getCurrentRowGrouping().getIObjectOrderIndex(nextRowObject);
	  if(y>e.getY())
	  row--;
	  IObject rowObject = matrix.getCurrentRowGrouping().getIObjectAt(row);
	*/

	if (getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_DRAG
	    || getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_ZOOMAREA){
	    guide.noRect(e.getX(), e.getY());
	}

	if (getView().getMouseState() == InfomatView.MOUSE_SELECT_PIXEL){
	    PixelFrame pixelFrame = getView().getInfomatModel().getPixelFrame();
	    if(pixelFrame!=null){

		/*
		int x = e.getX();
		int y = e.getY();
		SparsePixel sp = getPixel(x,y);
		pixelFrame.setCurrentInfo(sp);
		*/

		int x = e.getX();
		int y = e.getY();
		setInfoPixelFrame(x,y);

		int selectionState = pixelFrame.getSelectionState();
		if(selectionState == PixelFrame.SELECTION_STATE_SINGLE){

		}else if(selectionState == PixelFrame.SELECTION_STATE_AREA){


		    /*
		    IObjectGroup rowGroup = matrix.getRowIObjectSet().getCurrentGrouping().subGroup(dragStartRowObject, rowObject);
		    IObjectGroup colGroup = matrix.getColumnIObjectSet().getCurrentGrouping().subGroup(dragStartColObject, colObject);
		    System.out.println("rowGroup: "+rowGroup.numberOfIObjects());
		    System.out.println("colGroup: "+colGroup.numberOfIObjects());

		    Vector vec = matrix.subMatrix(rowGroup, colGroup).getAllCells();
		    pixelFrame.select(vec);
		    */

		    setSelectAreaPixelFrame(rowObject, colObject);


		    if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_CELLS){
			
			guide.noRect(e.getX(), e.getY());

		    }else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_ROWS){
			guide.noRows(e.getX(), e.getY());
			
		    }else if(pixelFrame.getSelectWhat() == PixelFrame.SELECT_WHAT_COLUMNS){
			guide.noCols(e.getX(), e.getY());
			
		    }


		}
	    }

	}else{


	    if(getView().getMouseState() == InfomatView.MOUSE_REMOVE_ROWS){
		guide.noRows(e.getX(), e.getY());

	    }else if(getView().getMouseState() == InfomatView.MOUSE_REMOVE_COLUMNS){
		guide.noCols(e.getX(), e.getY());
	    }
	    getView().dragged(colObject, rowObject, overviewPicture);
	}
    }



    public void mouseEntered(MouseEvent e) {
	if (getView().getMouseState() == InfomatView.MOUSE_CHANGE_STATE_DRAG) {
	    setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
	}
	guide.enter();
    }

    public void mouseExited(MouseEvent e) {
	if(toolbarView==null)
	    return;
	toolbarView.setInfo(null, null);
	//view.getInfoPanel().setCurrentInfo(null, null);
	//	view.getPixelInfoFrame().setCurrentInfo(null, null);

	setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

	guide.exit();
    }


    /*
    public void mouseWheelMoved(MouseWheelEvent e) {
	int notches = e.getWheelRotation();

	if(notches != 0){
	    int x = e.getX();
	    int y = e.getY();
	    System.out.println("Scrolled " + notches);
	    System.out.println("x = " + x + ", y = " + y);

	    IObject colCenterObject = getClosestColumnIObject(x);
	    IObject rowCenterObject = getClosestRowIObject(y);

	    if(colCenterObject!=null && rowCenterObject!=null)
		getView().zoom(colCenterObject, rowCenterObject, notches);
	}
    }
    */



    public void paintComponent(Graphics g) {
	System.out.println("SparsePicture - paintComponent");
	super.paintComponent(g);

	Graphics2D g2 = (Graphics2D) g;

	Dimension d = getSize();
	g2.setColor(getBackground());
	g2.fillRect(0, 0, d.width, d.height);


	int totalOffset = 0;
	if(view.getRowGrid())
	    totalOffset -= partPictureArray.length;
	if(view.getColumnGrid())
	    totalOffset -= partPictureArray[0].length;
	if(view.getRowGrid() && view.getColumnGrid())
	    totalOffset = -1;

	// Draw 
	int currentOrigoX=0;
	int currentOrigoY=0;
	for(int rgi = 0; rgi<partPictureArray[0].length; rgi++){
	    for(int cgi = 0; cgi<partPictureArray.length; cgi++){
		SparsePartPicture spp = partPictureArray[cgi][rgi];
		if(spp.toSmall())
		    continue;
		if(spp!=null){
		    currentOrigoX = spp.getOrigoX();
		    if(!view.getColumnGrid())
			currentOrigoX += spp.getOffsetX(); 
		    currentOrigoY = spp.getOrigoY();
		    if(!view.getRowGrid())
			currentOrigoY += spp.getOffsetY(); 
		    g.drawImage(spp.getImage(), 
				currentOrigoX, currentOrigoY, null);
		}

	    }
	    // Row Grid
	    if(view.getRowGrid()){
		g2.setColor(getGridColor());
		g2.drawLine(0, currentOrigoY-1, 
			    getWidth() + totalOffset, currentOrigoY-1);
	    }
	}

	// Column Grid
	if(view.getColumnGrid()){
	    for(int cgi = 0; cgi<partPictureArray.length; cgi++){
		SparsePartPicture spp = partPictureArray[cgi][0];
		currentOrigoX = spp.getOrigoX();
		g2.setColor(getGridColor());
		g2.drawLine(currentOrigoX-1, 0, 
			    currentOrigoX-1, getHeight() + totalOffset);
	    }
	}

	if(overviewPicture)
	    drawZoomRect(g);

    }

    public void drawZoomRect(Graphics g){ 
	//	System.out.println("-------------drawZoomRect");

	Graphics2D g2 = (Graphics2D) g;

	g2.setColor(getZoomRectColor());
	//	g2.setColor(getGridColor());
	g2.setBackground(getBackground());
	g2.setXORMode(getBackground());


	int rowFrom = getView().getRowFrom();
	int rowTo = getView().getRowTo();
	int colFrom = getView().getColFrom();
	int colTo = getView().getColTo();

	int x1 = getClosestXPixelForColumn(colFrom);
	int x2 = getClosestXPixelForColumn(colTo);
	int y1 = getClosestYPixelForRow(rowFrom);
	int y2 = getClosestYPixelForRow(rowTo);

	if(x2==-1)
	    x2=getSize().width-1;
	if(y2==-1)
	    y2=getSize().height-1;
		
	/*
	  g2.drawLine(x1, y1, x1, y2);
	  g2.drawLine(x2, y1, x2, y2);
	  g2.drawLine(x1, y1, x2, y1);
	  g2.drawLine(x1, y2, x2, y2);
	*/
		
	g2.drawRect(x1, y1, x2-x1, y2-y1);
    }


    public void update(Observable o, Object arg) {
	repaint();
    }

    public void saveIMatrixPicture(File file){
	BufferedImage image = new BufferedImage(imageDimension.width, 
						imageDimension.height,
						BufferedImage.TYPE_INT_RGB);
	Graphics2D g2 = image.createGraphics();
	g2.setBackground(getBackground());
	g2.setColor(getBackground());
	g2.fillRect(0,0,imageDimension.width,imageDimension.height);

	paintComponent((Graphics) g2);

	try {
	    ImageIO.write(image, "png", file);
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }
    
    public Guide getGuide(){
	return guide;
    }

    /*
    public void componentHidden(ComponentEvent e) {
	//	getGuide().exit();
    }

    public void componentShown(ComponentEvent e) {
	//	getGuide().enter();
    }
    */

}
