package mro.util.doublehashmap;

import java.util.*; 

/**
 * A DoubleHashMap has two indexes. 
 * There is only one copy of each index value.
 *
 * @version          080429
 * @author           Magnus Rosell
 */
public class GenericOrderedDoubleHashMap<F, S, V> 
    extends GenericDoubleHashMap<F, S, V>{

    private Vector<F> firstIndexOrderVector = new Vector<F>();
    private Vector<S> secondIndexOrderVector = new Vector<S>();

    public GenericOrderedDoubleHashMap(){}

    /**
     * Constructor. Use this when the first and second index 
     * has the same class as in these cases the invert()-method 
     * becomes faster.
     */
    public GenericOrderedDoubleHashMap(Class<F> fClass, Class<S> sClass, 
				       Class<V> vClass){ 
	super(fClass, sClass, vClass);
    }

    public GenericOrderedDoubleHashMap(GenericOrderedDoubleHashMap<F, S, V> map){
	super(map);
	firstIndexOrderVector = new Vector<F>(map.firstIndexOrderVector);
	secondIndexOrderVector = new Vector<S>(map.secondIndexOrderVector);
    }

    public GenericOrderedDoubleHashMap(GenericDoubleHashMap<F, S, V> map){
	super(map);
	firstIndexOrderVector = new Vector<F>(map.firstIndexSet);
	secondIndexOrderVector = new Vector<S>(map.secondIndexSet);
    }


    /**
     * Adds a Object "value" indexed with the Object:s
     * "first" and "second". The index objects are added in order.
     * @param first
     * @param second
     * @param value
     */
    public void put(F first, S second, V value){

	if(firstIndexSet.add(first))
	    firstIndexOrderVector.add(first);

	if(secondIndexSet.add(second))
	    secondIndexOrderVector.add(second);
		
	putIntoFirstMap(first, second, value, firstMap);
	putIntoSecondMap(second, first, value, secondMap);

	size++;
    }

    public GenericOrderedDoubleHashMap<S, F, V> invert(){
	GenericDoubleHashMap<S, F, V> newM = super.invert();
	GenericOrderedDoubleHashMap<S, F, V> newMap = 
	    new GenericOrderedDoubleHashMap<S, F, V>(newM);
	newMap.firstIndexOrderVector = secondIndexOrderVector;
	newMap.secondIndexOrderVector = firstIndexOrderVector;
	return newMap;
    }

    public void sortFirstIndex(Comparator<? super F> comp){
	Collections.sort(firstIndexOrderVector, comp);
    }

    public void sortSecondIndex(Comparator<? super S> comp){
	Collections.sort(secondIndexOrderVector, comp);
    }

    public Vector<F> getFirstIndexOrderVector(){
	return new Vector<F>(firstIndexOrderVector);
    }

    public Vector<S> getSecondIndexOrderVector(){
	return new Vector<S>(secondIndexOrderVector);
    }

    /*
    public V[][] getArray(){
	Vector<F> fis = getFirstIndexOrderVector();
	Vector<S> sis = getSecondIndexOrderVector();
	V[][] ret = new V[fis.size()][sis.size()];
	
	for(int f = 0; f<fis.size(); f++){
	    F fi = fis.elementAt(f);
	    for(int s = 0; s<sis.size(); s++){
		S si = sis.elementAt(s);
		V v = get(fi, si);
		if(v!=null)
		    ret[f][s] = v;
	    }
	}
	return ret;
    }
    */

    public void print(){
	Vector<F> fis = getFirstIndexOrderVector();
	Vector<S> sis = getSecondIndexOrderVector();
	
	for(int f = 0; f<fis.size(); f++){
	    F fi = fis.elementAt(f);
	    for(int s = 0; s<sis.size(); s++){
		S si = sis.elementAt(s);
		V v = get(fi, si);
		System.out.print(v);
	    }
	    System.out.println();
	}
	System.out.println();
    }




}
