package mro.util.measures;

import mro.util.*;

import infomat.vectorspace.IObjectGroup;

import java.io.*;
import java.util.*; 

/**
 * Average of a measure.
 *
 * @version          080322
 * @author           Magnus Rosell
 */
public class AverageMeasure extends Measure{

    /**
     * The vector of Measure:s.
     */
    Vector<Measure> measureVector = new Vector<Measure>();

    /**
     * The standard deviation.
     */ 
    double std;

    public AverageMeasure(){}

    public AverageMeasure(String name, String description){
	this.name=name;
	this.description=description;
    }


    /**
     * Copy Constructor. Deep (the measure vector is duplicated 
     * with new Measure:s).
     */
    public AverageMeasure(AverageMeasure am2){
	this.name=am2.name;
	this.value=am2.value;
	this.std=am2.std;
	this.description=am2.description;
	this.stamp=am2.getStamp();
	this.measureVector=new Vector<Measure>();
	for(int i=0; i<am2.measureVector.size(); i++)
	    this.measureVector.add(new Measure(am2.measureVector.elementAt(i)));
    }

    /**
     * Constructs a AverageMeasure consisting of only the given Measure.
     * Has to be used for starting adding Measure:s.
     */
    public AverageMeasure(Measure m){
	this.name=m.name;
	this.value=m.value;
	this.description=m.description;
	this.stamp=m.getStamp();
	this.measureVector=new Vector<Measure>();
	this.measureVector.add(m);
    }


    public boolean addMeasure(Measure m){
	if(m.getName().compareTo(name)!=0)
	    return false;
	if(m instanceof AverageMeasure)
	    measureVector.addAll(((AverageMeasure) m).measureVector);
	else
	    measureVector.add(m);
	return true;
    }

    protected void sortMeasureVectorInStampOrder(){
	Collections.sort(measureVector, new MeasureStampComparator());
    }

    /**
     * Constructs a AverageMeasure containing the difference between this and 
     * an other AverageMeasure for each Measure in the Vector. (this-other)
     */
    public AverageMeasure difference(AverageMeasure other){
	if(this.measureVector.size()!=other.measureVector.size())
	    return null;
	
	other.sortMeasureVectorInStampOrder();


	AverageMeasure am = new AverageMeasure(this);

	am.sortMeasureVectorInStampOrder();

	for(int i=0; i<am.measureVector.size(); i++){
	    am.measureVector.set(i, am.measureVector.elementAt(i).difference(other.measureVector.elementAt(i)));
	}
	return am;
    }

    /**
     * Constructs a AverageMeasure containing the normalized 
     * difference between this and an other AverageMeasure
     * for each Measure in the Vector. (this-other)/other
     */
    public AverageMeasure normDifference(AverageMeasure other){
	AverageMeasure am = difference(other);
	for(int i=0; i<am.measureVector.size(); i++){
	    Measure m = am.measureVector.elementAt(i);
	    Measure om = other.measureVector.elementAt(i);
	    m.value = m.value/om.value;
	    System.out.print(m.value+" ");
	}
	System.out.println();
	return am;
    }

    public AverageMeasure subtract(AverageMeasure other){
	if(this.measureVector.size()!=other.measureVector.size())
	    return null;
	
	other.sortMeasureVectorInStampOrder();
	AverageMeasure am = new AverageMeasure(this);
	am.sortMeasureVectorInStampOrder();
	for(int i=0; i<am.measureVector.size(); i++){
	    am.measureVector.set(i, am.measureVector.elementAt(i).subtract(other.measureVector.elementAt(i)));
	}
	return am;
    }

    public AverageMeasure divide(AverageMeasure other){
	if(this.measureVector.size()!=other.measureVector.size())
	    return null;
	
	other.sortMeasureVectorInStampOrder();
	AverageMeasure am = new AverageMeasure(this);
	am.sortMeasureVectorInStampOrder();
	for(int i=0; i<am.measureVector.size(); i++){
	    am.measureVector.set(i, am.measureVector.elementAt(i).divide(other.measureVector.elementAt(i)));
	}
	return am;
    }
	



    
    private void calculate(){
	value=0.0;
	std=0.0;
	for(int i=0; i<measureVector.size(); i++){
	    value += measureVector.elementAt(i).getValue();
	}
	value /= measureVector.size();


	std=0.0;
	for(int i=0; i<measureVector.size(); i++){
	    std += Math.pow(measureVector.elementAt(i).getValue()-value, 2);
	}
	std = Math.sqrt(std/measureVector.size());
    }

    public double getValue(){
	calculate();
	return value;
    }
    
    public double getSTD(){
	calculate();
	return std;
    }
    
    public void print(){
	calculate();
	System.out.println(name+": "+value+", "+std);
    }


    public void printXML(PrintWriter pw) throws IOException{
	calculate();
	pw.println("<am>");
	pw.print("<n>"+getName()+"</n>");
	pw.println("<v>"+getValue()+"</v>");
	pw.println("<std>"+getSTD()+"</std>");
	pw.println("<d>"+getDescription()+"</d>");
	pw.println("<stamp>"+getStamp()+"</stamp>");
	for(int i = 0; i < measureVector.size(); i++){
	    measureVector.get(i).printXML(pw);
	}
	pw.println("</am>");
    }

}


