package mro.util.measures;

import mro.util.*;

import infomat.vectorspace.IObjectGroup;

import java.io.*;
import java.util.*; 

/**
 * A measure.
 *
 * @version          061127
 * @author           Magnus Rosell
 */

// FIXA in- och utlsning av relativa IOBjectGroup
 
public class Measure{
    String name;
    double value;
    String description;
    IObjectGroup iogrelative;

    /**
     * An "identifier" for the Measure, for making it easier
     * to compare and manipulate Measure:s.
     */
    String stamp;

    public static boolean roundValue=false;
    static int roundFloatToNumOfDecimals=3;

    public Measure(){}

    public Measure(String name, double value, String description){
	this.name=name;
	this.value=value;
	this.description=description;
    }

    public Measure(String name, double value, 
		   String description, String stamp){
	this.name=name;
	this.value=value;
	this.description=description;
	this.stamp=stamp;
    }


    /**
     * Copy Constructor.
     */
    public Measure(Measure m2){
	this.name=m2.name;
	this.value=m2.value;
	this.description=m2.description;
	this.stamp=m2.stamp;
	this.iogrelative=m2.iogrelative;
    }


    /**
     * To read from XML files...
     */
    public Measure(String name, String value, 
		   String description, String stamp){
	double o=0.0;

	try{o = (Double) Double.parseDouble(value);}
	catch(Exception e2){
	    System.out.println("Problem: Measure: not a double!");
	    System.exit(0);
	}

	this.name=name;
	this.value=o;
	this.description=description;
	this.stamp=stamp;
    }
	
    public Measure(String name, double value, String description, IObjectGroup iogrelative){
	this.name=name;
	this.value=value;
	this.description=description;
	this.iogrelative=iogrelative;
    }

    /**
     * Constructs a Measure containing the difference between this and 
     * an other Measure. (this-other)
     */
    public Measure difference(Measure other){
	Measure m = new Measure(this);
	m.value = m.value - other.value;
	m.name = "Differenece in "+m.name;
	m.description = "Differenece in "+m.description;
	return m;
    }

    /**
     * Constructs a Measure containing the normalized 
     * difference between this and an other Measure. (this-other)/other
     */
    public Measure normDifference(Measure other){
	Measure m = difference(other);
	m.value = m.value/other.value;
	return m;
    }

    public Measure subtract(Measure other){
	Measure m = new Measure(this);
	m.value = m.value - other.value;
	return m;
    }

    public Measure divide(Measure other){
	Measure m = new Measure(this);
	m.value = m.value / other.value;
	return m;
    }
	
    public IObjectGroup getRelativeIObjectGroup(){
	return iogrelative;
    }

    public void setValue(double value){
	this.value=value;
    }

    public double getValue(){
	return value;
    }


    public String getName(){
	return name;
    }

    public String getDescription(){
	return description;
    }

    public void setDescription(String description){
	this.description=description;
    }

    public String getStamp(){
	return stamp;
    }

    public void setStamp(String stamp){
	this.stamp=stamp;
    }

    public void printXML(PrintWriter pw) throws IOException{
	pw.println("<m>");
	pw.print("<n>"+getName()+"</n>");
	pw.println("<v>"+getValue()+"</v>");
	pw.println("<d>"+getDescription()+"</d>");
	pw.println("<stamp>"+getStamp()+"</stamp>");
	pw.println("</m>");
    }

}


